// Digital_Scope 0.91 by Earl Colby Pottinger. 
// Partly based on code found in the Beos CDROM 
// Partly based on instructions found in "Programming the Be Operating System" By Dan Parks Sydow. 
// Additional help by Larry Baydak in debugging and coding/commenting style. 
// GUI help by M4RCONE@XS4ALL.NL - Thanks.

// #include "Digital_Scope_Includes.h"
// #include "Digital_Scope_Menus.h"

// #include "Digital_Scope_View.h"
// #include "Digital_Display_View.h"
// #include "Digital_Colour_View.h"

// #include "Digital_Scope_Window.h"
// #include "Digital_Colour_Window.h"

// #include "Digital_Scope_World.h"

// #include "Digital_Scope_View.cpp"
// #include "Digital_Display_View.cpp"
// #include "Digital_Colour_View.cpp"

// #include "Digital_Scope_Window.cpp"
// #include "Digital_Colour_Window.cpp"

// #include "Digital_Scope_World.cpp"

// Digital_Scope_Includes.h 0.91 by Earl Colby Pottinger. 
// Needed for all basic applications.
#include <Application.h>  // Used for all programs.
#include <InterfaceKit.h>
#include <Window.h>       // Used to create windows on the desktop.
#include <View.h>         // Used to create views inside a window.
#include <Alert.h>        // Used to open BAlert Windows.

// Needed to exchange messages.
#include <Messenger.h>
#include <Message.h>
#include <Roster.h>

// Needed to create and use menus.
#include <Menu.h>
#include <MenuBar.h>
#include <MenuItem.h>

// Needed to create and use Strings & BStrings.
#include <string.h> // Posix string includes.
#include <String.h> // BString includes.
#include <StringView.h>

// Needed for input/output to devices.
#include <stdio.h>  // Basic input/output.
#include <stdlib.h> // Standard library.

// Needed to load, display, and save text files.
#include <StorageKit.h>
#include <Entry.h>
#include <Path.h>
#include <FilePanel.h>

#include <fcntl.h>
#include <unistd.h>

// Includes/Defines from Parallel driver code.
#include <iostream.h>
#include "paraclass.h"

uint32 lpt = 0x378; // change this to whatever your parallel port address is, it's 0x378 on most comps.

#define PO (lpt + 0) // parallel port I/O base address
#define PS (lpt + 1) // parallel port status address
#define PC (lpt + 2) // parallel port control address

ParPort *port; // Parallel port structure.

// Digital_Scope_Menus.h 0.91 by Earl Colby Pottinger. 
// Constants Messages for menu commands.
const uint32 MENU_FILE_NEW       = 'New_'; // Clear all buffers.
const uint32 MENU_FILE_OPEN      = 'Open'; // Load a file into buffer.
const uint32 MENU_FILE_CLOSE     = 'Clse'; // Close loading a file into buffer.
const uint32 MENU_FILE_SAVE      = 'Save'; // Save buffer into a file.
const uint32 MENU_FILE_SAVEAS    = 'Svas'; // Save buffer into a file.
const uint32 MENU_FILE_PAGESETUP = 'Page'; // Setup page margins.
const uint32 MENU_FILE_PRINT     = 'Prnt'; // Print data.
const uint32 MENU_FILE_PREFS     = 'Pref'; // Program preferences.
const uint32 MENU_FILE_PREFS_DISPLAY  = 'Prf0'; // Set colour of the Display background.
const uint32 MENU_FILE_PREFS_GRID     = 'Prf1'; // Set colour of the Grid lines.
const uint32 MENU_FILE_PREFS_BULLsEYE = 'Prf2'; // Set colour of the Bull's Eye lines.
const uint32 MENU_FILE_PREFS_DIGITAL  = 'Prf3'; // Set colour of the Digital Data lines.
const uint32 MENU_FILE_PREFS_ZERO     = 'Prf4'; // Set colour of the Digital Zero Level lines.
const uint32 MENU_FILE_PREFS_ANALOG1  = 'Prf5'; // Set colour of the Analog 1 Data.
const uint32 MENU_FILE_PREFS_ANALOG2  = 'Prf6'; // Set colour of the Analog 2 Data.
const uint32 MENU_FILE_PREFS_ANALOG3  = 'Prf7'; // Set colour of the Analog 3 Data.
const uint32 MENU_FILE_PREFS_ANALOG4  = 'Prf8'; // Set colour of the Analog 4 Data.
const uint32 MENU_FILE_ABOUT     = 'What'; // About the program.
const uint32 MENU_FILE_QUIT      = 'Quit'; // Exit program.

const uint32 MENU_EDIT_UNDO    = 'Undo'; // Reverse last changes.
const uint32 MENU_EDIT_CUT     = 'Cut_'; // Move selected data to clipboard.
const uint32 MENU_EDIT_COPY    = 'Copy'; // Copy selected data to clipboard.
const uint32 MENU_EDIT_PASTE   = 'Pste'; // Paste clipboard to present position.
const uint32 MENU_EDIT_SELECT  = 'All_'; // Select all data.
const uint32 MENU_EDIT_FIND    = 'Find'; // Find data sequence.
const uint32 MENU_EDIT_REPLACE = 'Rplc'; // Replace data sequence.

const uint32 MENU_COUNT_UP       = 'PrtU'; // Count up parallel output.
const uint32 MENU_COUNT_DOWN     = 'PrtD'; // Count down parallel output
const uint32 MENU_READ_PORT      = 'PrtR'; // Read a value off parallel port's inputs.
const uint32 MENU_WRITE_PORT_0   = 'Prt0'; // Write a value of '0' on parallel port's outputs.
const uint32 MENU_WRITE_PORT_85  = 'Prt8'; // Write a value of '85' on parallel port's outputs.
const uint32 MENU_WRITE_PORT_170 = 'Prt1'; // Write a value of '170' on parallel port's outputs.
const uint32 MENU_WRITE_PORT_255 = 'Prt2'; // Write a value of '255' on parallel port's outputs.
const uint32 MENU_SCAN_PLOT      = 'SPlt'; // Sample and plot data.
const uint32 MENU_FAKE_SCAN      = 'RPlt'; // Fake a data scan.
const uint32 MENU_RESTORE        = 'Rstr'; // Restore default settings.
const uint32 MENU_GRID_SWCTH     = 'Grid'; // Change display of grid modes.
const uint32 MENU_BULLS_EYE      = 'Bull'; // Turn display of bull's eye on/off
const uint32 MENU_NORMAL         = 'Norm'; // Normal scaling of analog plotting.
const uint32 MENU_MAX            = 'Maxm'; // Maxinize scaling of analog plotting.

const uint32 APP_COUNT_UP   = 'PrtU'; // Count up parallel output.
const uint32 APP_COUNT_DOWN = 'PrtD'; // Count down parallel output.
const uint32 APP_READ_PORT  = 'PrtR'; // Read values off parallel port's inputs.
const uint32 APP_WRITE_PORT = 'Wprt'; // Write to port.
const uint32 APP_SCAN_PLOT  = 'SPlt'; // Sample and plot data.

const uint32 MENU_TIMING_1 = 'Tmg0'; // 1 sample per second.
const uint32 MENU_TIMING_2 = 'Tmg1'; // 10 samples per second.
const uint32 MENU_TIMING_3 = 'Tmg2'; // 100 samples per second.
const uint32 MENU_TIMING_4 = 'Tmg3'; // 1000 samples per second.
const uint32 MENU_TIMING_5 = 'Tmg4'; // 10000 samples per second.
const uint32 MENU_TIMING_6 = 'Tmg5'; // 100000 samples per second.
const uint32 MENU_TIMING_7 = 'Tmg6'; // MAX samples per second.

const uint32 MENU_DIGIT2  = 'Dgt2'; // Read 2 digital lines in.
const uint32 MENU_DIGIT4  = 'Dgt4'; // Read 4 digital lines in.
const uint32 MENU_DIGIT6  = 'Dgt6'; // Read 6 digital lines in.
const uint32 MENU_DIGIT8  = 'Dgt8'; // Read 8 digital lines in.
const uint32 MENU_BAR_ADC10 = 'Scn0'; // Read 10 LED Bar ADC in.
const uint32 MENU_BAR_ADC08 = 'Scn8'; // Read 8 LEDs Bar ADC in.
const uint32 MENU_2_4_ADC = 'Scn2'; // Read 2 Flash 4-Bit ADCs.
const uint32 MENU_1_8_ADC = 'Scn3'; // Read 1 Flash 8-Bit ADC.
const uint32 MENU_SCAN_1  = 'Scn4'; // Scan for one 8 bit analog value.
const uint32 MENU_SCAN_2  = 'Scn5'; // Scan for two 8 bit analog values.
const uint32 MENU_SCAN_4  = 'Scn6'; // Scan for four 8 bit analog values.

const uint32 MENU_TRIGGER_0 = 'Trg0'; // Input 0 source of triggering scan.
const uint32 MENU_TRIGGER_1 = 'Trg1'; // Input 1 source of triggering scan.
const uint32 MENU_TRIGGER_2 = 'Trg2'; // Input 2 source of triggering scan.
const uint32 MENU_TRIGGER_3 = 'Trg3'; // Input 3 source of triggering scan.
const uint32 MENU_TRIGGER_4 = 'Trg4'; // Input 4 source of triggering scan.
const uint32 MENU_TRIGGER_5 = 'Trg5'; // Input 5 source of triggering scan.
const uint32 MENU_TRIGGER_6 = 'Trg6'; // Input 6 source of triggering scan.
const uint32 MENU_TRIGGER_7 = 'Trg7'; // Input 7 source of triggering scan.

const uint32 MENU_NO_TRIGGER      = 'Trgn'; // No Trigger needed.
const uint32 MENU_RISING_TRIGGER  = 'Trgr'; // Trigger scan if signal rises.
const uint32 MENU_FALLING_TRIGGER = 'Trgf'; // Trigger scan if signal falls.
const uint32 MENU_MID_TRIGGER     = 'Trgm'; // Trigger scan if signal cross mid value.
const uint32 MENU_VALUE_TRIGGER   = 'Trgv'; // Trigger scan if signal equals value.

const uint32 MENU_SINGLE_SHOT     = 'Trgs'; // Run only once when trigger conditions meet.
const uint32 MENU_EVERY_TRIGGER   = 'Trge'; // Run everytime when trigger conditions meet.

const uint32 MENU_PULSE_OFF = 'Pls0'; // Sends a pulse message to BApplication to turn off Pulses.
const uint32 MENU_PULSE_0   = 'Pls1'; // Sends a pulse message to BApplication, every .1 seconds.
const uint32 MENU_PULSE_1   = 'Pls2'; // Sends a pulse message to BApplication, every 1 seconds.
const uint32 MENU_PULSE_10  = 'Pls3'; // Sends a pulse message to BApplication, every 10 seconds.
const uint32 MENU_PULSE_100 = 'Pls4'; // Sends a pulse message to BApplication, every 100 seconds.

const uint32 APP_PULSE_OFF = 'Pls0'; // Sends a pulse message to BApplication to turn off Pulses.
const uint32 APP_PULSE_0   = 'Pls1'; // Sends a pulse message to BApplication, every .1 seconds.
const uint32 APP_PULSE_1   = 'Pls2'; // Sends a pulse message to BApplication, every 1 seconds.
const uint32 APP_PULSE_10  = 'Pls3'; // Sends a pulse message to BApplication, every 10 seconds.
const uint32 APP_PULSE_100 = 'Pls4'; // Sends a pulse message to BApplication, every 100 seconds.

const uint32 MENU_DEBUG_OFF = 'Dbg0'; // DeBug off.
const uint32 MENU_DEBUG_1   = 'Dbg1'; // DeBug .
const uint32 MENU_DEBUG_2   = 'Dbg2'; // DeBug .
const uint32 MENU_DEBUG_3   = 'Dbg3'; // DeBug .
const uint32 MENU_DEBUG_4   = 'Dbg4'; // DeBug .
const uint32 MENU_DEBUG_5   = 'Dbg5'; // DeBug .
const uint32 MENU_DEBUG_6   = 'Dbg6'; // DeBug .
const uint32 MENU_DEBUG_7   = 'Dbg7'; // DeBug .
const uint32 MENU_DEBUG_8   = 'Dbg8'; // DeBug .
const uint32 MENU_DEBUG_9   = 'Dbg9'; // DeBug .

const uint32 SLIDER_1 = 'Sld1', SLIDER_2 = 'Sld2', SLIDER_3 = 'Sld3', SLIDER_4 = 'Sld4', SLIDER_5 = 'Sld5', SLIDER_6 = 'Sld6'; // Slider BMessages
const uint32 SLIDER_7 = 'Sld7', SLIDER_8 = 'Sld8', SLIDER_9 = 'Sld9', SLIDER_0 = 'Sld0', SLIDER_A = 'SldA'; // Used to adjust analog display

const uint32 REFRESH_SCREEN  = 'Scrn';
const uint32 APP_SLEEP       = 'Aslp'; // Go to sleep for one second
const uint32 MENU_CENTER  = 'Cntr'; // Center all analog displays
const uint32 MENU_OFFSET  = 'OffS'; // Offset all analog displays
const uint32 REFRESH_DISPLAY = 'Rfhd'; // Force redraw of display

const uint32 COLOUR_CHANGED = 'ClCh'; // New colour picked.
const uint32 COLOUR_SET = 'Cset';     // Set colour of prefs control
const uint32 COLOUR_OK = 'ClOk';      // Finished changing colour

//Constants needed. 
const char *APP_SIGNATURE="application/x-gui.Parallel_Port_Digital_Scope"; // Application's signature. Check match's resource.

//Global Variables needed.
bool  flag1 = false, flag2 = false, flag3 = false, flag4 = false, LoopFlag = false, BullsEye = true;
uint8 data1 = 0, data2 = 0, data3 = 0, data4 = 0, scan = 0, data = 0, color = 0;
int   DeBug = 0, Display = 0, HorzScale = 1, HorzOffset0 = 0, HorzOffset1 = 0, HorzOffset2 = 0, DataWindow = 0, DataOffset = 0, GridMode = 2;
int   TriggerMode = 0, TriggerValue = 0, ScanMode = 1, Source = 0, InputType = 0, Timing = 1000;
bigtime_t TimeBase;
float scaling1 = .2, scaling2 = .2, scaling3 = .2, scaling4 = .2, X, Y, Z;
float offset1 = 80.0, offset2 = 160.0, offset3 = 240.0, offset4 = 320.0;

rgb_color blackColor    = {  0,  0,  0,255};
rgb_color gridColor     = {128,128,128,255};
rgb_color bulleyeColor  = { 64, 64, 64,255};
rgb_color digitalColor  = {128,  0,128,255};
rgb_color zeroColor     = {192,192,192,255};
rgb_color analog1Color  = {255,  0,  0,255};
rgb_color analog2Color  = {  0,255,  0,255};
rgb_color analog3Color  = {  0,  0,255,255};
rgb_color analog4Color  = {128,  0,128,255};
rgb_color displayColor  = {127,127,127,255};

int Mask[]={1,2,4,8,16,32,64,128};

// Internal buffer.
long n, m, count;  // General counters.
long FileLenght=0; // Number of bytes in file input buffer.
char TextBuffer[999];  // Text buffer to convert number variables.  Print lines in columns

uint8 InputBuffer1[1028]; // Buffers to store scan in from port to memory.
uint8 InputBuffer2[1028]; // Extra size is incase of overruns during the convertion process.
uint8 InputBuffer3[1028];
uint8 InputBuffer4[1028];

// Objects needed.
BFile      File_Object;     // BFile object
BFilePanel *OpenFilePanel = NULL; // Define object to create open panels from.
BFilePanel *SaveFilePanel = NULL; // Define object to create save panels from.
BPath      file_path;  //
BEntry     b_entry;

const char *file_name;
entry_ref  file_ref;
status_t   File_Error; // The error/return code
size_t     Bytes_Read; // Number of bytes read in from file.
off_t      File_Size;  // Length of file.

int    ref, refs;  // Index list of files passed
bool   LoadFlag = false; // True to load file.
bool   SaveFlag = false; // True to save file.
BAlert *alert;   // Create a pointer (*alert) to an alert window
long   result=0; // Returned value from the BAlert::Go() member function.

BColorControl *aColor;  // Colour Control Object.

// Digital_Scope_View.h
class Digital_Scope_View : public BView {                   // Define the class object Digital_Scope_View.
 public:       Digital_Scope_View(BRect frame, char *name); // Constructor for a view inside a window.
  virtual void Draw(BRect updateRect); } ;                  // Called by the OS when 'aWindow->Show()' used.

// Digital_Display_View.h
class Digital_Display_View : public BView {                   // Define the class object Digital_Display_View.
 public:       Digital_Display_View(BRect frame, char *name); // Constructor for a view inside a window.
  virtual void Draw(BRect updateRect); } ;                    // Called by the OS when 'aWindow->Show()' used.

// Digital_Colour_View.h
class Digital_Colour_View : public BView {                   // Define the class object Digital_Colour_View.
 public:       Digital_Colour_View(BRect frame, char *name); // Constructor for a view inside a window.
  virtual void Draw(BRect updateRect); } ;                   // Called by the OS when 'aWindow->Show()' used.

// Digital_Scope_Window.h
class Digital_Scope_Window : public BWindow {      // Define the class object Digital_Scope_Window.
 public:       Digital_Scope_Window(BRect frame);  // Constructor for a window thread.
  virtual bool QuitRequested();                    // Add function is close box is clicked.
  virtual void MessageReceived(BMessage *message); // Setup to recieve menu & windows messages.
  virtual void FrameMoved(BPoint new_position);    // Window moved to new position.
  BSlider     *slider1, *slider2, *slider3, *slider4, *slider5, *slider6, *slider7, *slider8, *slider9, *slider0, *sliderA;
  BMenuItem   *menuitem;
 private:
  BMenuBar     *menubar; } ;                       // Define object to hold all menu structures.

// Digital_Colour_Window.h
class Digital_Colour_Window : public BWindow {     // Define the class object Digital_Colour_Window .
 public:       Digital_Colour_Window(BRect frame); // Constructor for a window thread.
  virtual bool QuitRequested();                    // Add function is close box is clicked.
  virtual void MessageReceived(BMessage *message); // Setup to recieve menu & windows messages.
  virtual void FrameMoved(BPoint new_position); } ;

// Digital_Scope_World.h
class Digital_Scope_Application : public BApplication { // Define the class object Digital_Scope_Application.
 public:       Digital_Scope_Application();             // Constructor for an application thread.
               ~Digital_Scope_Application();            // Deconstructor for an application thread.
  virtual void ReadyToRun();                            // Called before BLooper starts 
  virtual void AboutRequested();                        // Recieved a B_ABOUT_REQUESTED message.
  virtual void MessageReceived(BMessage *message);      // Recieves system or control messages.
  virtual void RefsReceived(BMessage *message);         // Recieves entry_ref messages from file panel.
  virtual void Pulse(); } ;                             // Recieved a B_PULSE message.

// Variables needed with windows.
BPoint         Window_Location1; // Position of Digital_Scope_Window on screen.
BPoint         Window_Location2; // Position of Digital_Scope_Window on screen.

BRect            aWindow1_Rect; // Dimension of window.
BRect            aWindow2_Rect; // Dimension of window.

BRect             aView1_Rect; // Dimensions of view inside window.
BRect             aView2_Rect; // Dimension of scope's display inside window.
BRect             aView3_Rect; // Dimension of colour control's view inside window.

Digital_Scope_View    *aView1; // Create a pointer(*aView1) to an object(Digital_Scope_View).
Digital_Display_View  *aView2; // Create a pointer(*aView2) to an object(Digital_Display_View).
Digital_Colour_View   *aView3; // Create a pointer(*aView3) to an object(Digital_Colour_View).

Digital_Scope_Window  *aWindow1; // Create a pointer(*aWindow1) to an object(Digital_Scope_Window).
Digital_Colour_Window *aWindow2; // Create a pointer(*aWindow2) to an object(Digital_Colour_Window).

// Digital_Scope_View.cpp
Digital_Scope_View::Digital_Scope_View(BRect rect, char *name) // Define 'Digital_Scope_View' constructor call.
             : BView(rect, name, B_FOLLOW_ALL, B_WILL_DRAW) { } ;

void Digital_Scope_View::Draw(BRect updateRect) { // Draws in view when show is called. 
 SetFont(be_plain_font); SetFontSize(12); // Set font outline to plain. Set font size.
 X=19.0; Y=19.0; MovePenTo(BPoint(X,Y)); SetHighColor(blackColor); // Draw outline of display area. Set pen to Black.
 StrokeLine(BPoint(X+802,Y),B_SOLID_HIGH); StrokeLine(BPoint(X+802,Y+402),B_SOLID_HIGH); // Outline display area.
 StrokeLine(BPoint(X,Y+402),B_SOLID_HIGH); StrokeLine(BPoint(X,Y),B_SOLID_HIGH);
 X=20.0; Y=440.0;  MovePenTo(BPoint(X,Y)); // Move pen to top of window.
 if (Timing) {
    sprintf(TextBuffer,"Samples per Seconds: %d",1000000/Timing); DrawString(TextBuffer); }
 else {
    DrawString("Maxnium sampling possible."); }

 X=20.0; Y=Y+15.0; MovePenTo(BPoint(X,Y)); // Move pen to next line.
 if (InputType == 0) { DrawString("Read 2 digital lines in."); }
 if (InputType == 1) { DrawString("Read 4 digital lines in."); }
 if (InputType == 2) { DrawString("Read 6 digital lines in."); }
 if (InputType == 3) { DrawString("Read 8 digital lines in."); }

 if (InputType == 4)  { DrawString("Read 8 LEDs Bar ADC in."); }
 if (InputType == 10) { DrawString("Read 10 LEDs Bar ADC in."); }

 if (InputType == 5) { DrawString("Read 2 Flash 4-Bit ADCs."); }
 if (InputType == 6) { DrawString("Read 1 Flash 8-Bit ADC."); }
 if (InputType == 7) { DrawString("Scan 1 analog 8 bit line."); }
 if (InputType == 8) { DrawString("Scan 2 analog 8 bit lines."); }
 if (InputType == 9) { DrawString("Scan 4 analog 8 bit lines."); }

// X=20.0; Y=Y+15.0; MovePenTo(BPoint(X,Y)); // Move pen to next line.
// sprintf(TextBuffer,"Trigger source. Line: %d ",Source); DrawString(TextBuffer);

// X=20.0; Y=Y+15.0; MovePenTo(BPoint(X,Y)); // Move pen to next line.
// if (ScanMode == 1) { DrawString("Run only once."); }
// if (ScanMode == 2) { DrawString("Run everytime."); }

// X=20.0; Y=Y+15.0; MovePenTo(BPoint(X,Y)); // Move pen to next line.
// if (TriggerMode == 0) { DrawString("Scan at once. No Trigger."); }
// if (TriggerMode == 1) { DrawString("Trigger scan if signal falls."); }
// if (TriggerMode == 2) { DrawString("Trigger scan if signal cross mid value."); }
// if (TriggerMode == 3) { DrawString("Trigger scan if signal equals value."); }
 } // End of Digital_Scope_View

// Digital_Display_View.cpp
Digital_Display_View::Digital_Display_View(BRect rect, char *name) // Define 'Digital_Display_View' constructor call.
             : BView(rect, name, B_FOLLOW_ALL, B_WILL_DRAW) { };

void Digital_Display_View::Draw(BRect updateRect) { // Draws in view when show is called. 
 SetViewColor(displayColor);
 if ((Display < 1)|(Display > 4)) { // Valid to display grid
     if (GridMode) {
        SetHighColor(gridColor); // Draw grid inside display area. Set pen to grid colour.
        if (GridMode == 7) { GridMode = 0; }
        if (GridMode == 1) {
           for (Y=0.0; Y<401.0; Y=Y+20.0) { MovePenTo(BPoint(0.0,Y)); StrokeLine(BPoint(800.0,Y),B_SOLID_HIGH); }
           for (X=0.0; X<801.0; X=X+20.0) { MovePenTo(BPoint(X,0.0)); StrokeLine(BPoint(X,400.0),B_SOLID_HIGH); } }
        if (GridMode == 2) {
           for (Y=0.0; Y<401.0; Y=Y+40.0) { MovePenTo(BPoint(0.0,Y)); StrokeLine(BPoint(800.0,Y),B_SOLID_HIGH); }
           for (X=0.0; X<801.0; X=X+40.0) { MovePenTo(BPoint(X,0.0)); StrokeLine(BPoint(X,400.0),B_SOLID_HIGH); } }
        if (GridMode == 3) {
           for (X=0.0; X<801.0; X=X+20.0) { MovePenTo(BPoint(X,0.0)); StrokeLine(BPoint(X,400.0),B_SOLID_HIGH); } }
        if (GridMode == 4) {
           for (X=0.0; X<801.0; X=X+40.0) { MovePenTo(BPoint(X,0.0)); StrokeLine(BPoint(X,400.0),B_SOLID_HIGH); } }
        if (GridMode == 5) {
           for (Y=0.0; Y<401.0; Y=Y+20.0) {
               for (X=0.0; X<801.0; X=X+20.0) {
                   MovePenTo(BPoint(X-2.0,Y)); StrokeLine(BPoint(X+2.0,Y),B_SOLID_HIGH);
                   MovePenTo(BPoint(X,Y-2.0)); StrokeLine(BPoint(X,Y+2.0),B_SOLID_HIGH); } } }
        if (GridMode == 6) {
           for (Y=0.0; Y<401.0; Y=Y+20.0) {
               for (X=0.0; X<801.0; X=X+20.0) {
                   MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X,Y),B_SOLID_HIGH); } } } }
     if (BullsEye) {
        SetHighColor(bulleyeColor); // Draw bull's eye inside display area. Set pen to bull's eye colour.
        MovePenTo(BPoint(0.0,200.0)); StrokeLine(BPoint(800.0,200.0),B_SOLID_HIGH);
        MovePenTo(BPoint(400.0,0.0)); StrokeLine(BPoint(400.0,400.0),B_SOLID_HIGH);
        for (X=0.0; X<801.0; X=X+20.0) { 
            MovePenTo(BPoint(X,197.0)); StrokeLine(BPoint(X,203.0),B_SOLID_HIGH);
            MovePenTo(BPoint(X+10.0,199.0)); StrokeLine(BPoint(X+10.0,201.0),B_SOLID_HIGH); }
        for (Y=0.0; Y<401.0; Y=Y+20.0) { 
            MovePenTo(BPoint(397.0,Y)); StrokeLine(BPoint(403.0,Y),B_SOLID_HIGH);
            MovePenTo(BPoint(399.0,Y+10.0)); StrokeLine(BPoint(401.0,Y+10.0),B_SOLID_HIGH); } } };

 if (Display == 1) { // Counting Up
    MovePenTo(BPoint(100.0,150.0)); SetFontSize(96); DrawString("Counting Up."); SetFontSize(12); };

 if (Display == 2) { // Counting Down
    MovePenTo(BPoint(100.0,150.0)); SetFontSize(96); DrawString("Counting Down."); SetFontSize(12); };

 if (Display == 3) { // Read the parallel port.
    SetFontSize(36); // Nice font size
    MovePenTo(BPoint(100.0,150.0)); sprintf(TextBuffer,"Parallel Port reads --> %d ",data1); DrawString(TextBuffer);
    MovePenTo(BPoint(100.0,200.0)); sprintf(TextBuffer,"  Status Port reads --> %d ",data2); DrawString(TextBuffer);
    MovePenTo(BPoint(100.0,250.0)); sprintf(TextBuffer," Control Port reads --> %d ",data3); DrawString(TextBuffer);
    SetFontSize(12); }; // Restore font size

 if (Display == 4) { // Write pattern to parallel port.
    MovePenTo(BPoint(100.0,150.0)); SetFontSize(48); sprintf(TextBuffer,"Writting Data value %d to Port.",data); DrawString(TextBuffer); SetFontSize(12); };

 if (Display == 5) {
    DataWindow = 800/HorzScale; DataOffset = ((1024-DataWindow)*HorzOffset0)/1024; // Read in and Plot data.
    
    if (InputType == 0) { // 2 Direct digital inputs, 1 bit each for 1 byte from the parallel port.
       X=0.0; Y=160.0; SetHighColor(zeroColor); // Set drawing colour
       MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+160.0; MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH);
       SetHighColor(digitalColor); data2 = InputBuffer1[DataOffset]; // Set drawing colour, Get first data element.
       for (n=DataOffset; n<DataWindow+DataOffset+1; n++) {
           data1 = data2; data2 = InputBuffer1[n+1]; Y=80.0;
           for (scan=0; scan<2; scan++) {
               if (Mask[scan] & data1) {
                  MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+HorzScale,Y),B_SOLID_HIGH);
                  if (!(Mask[scan] & data2)) {
                     StrokeLine(BPoint(X+HorzScale,Y+80.0),B_SOLID_HIGH); } }
               else {
                  MovePenTo(BPoint(X,Y+80.0)); StrokeLine(BPoint(X+HorzScale,Y+80.0),B_SOLID_HIGH);
                  if (Mask[scan] & data2) {
                     StrokeLine(BPoint(X+HorzScale,Y),B_SOLID_HIGH); } }
               Y=Y+160.0; }
           X=X+HorzScale; }}

    if (InputType == 1) { // 4 Direct digital inputs, 1 bit each for 1 byte from the parallel port.
       X=0.0; Y=80.0; SetHighColor(zeroColor); // Set drawing colour
       MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+80.0; MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+120.0;
       MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+80.0; MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH);
       SetHighColor(digitalColor); data2 = InputBuffer1[DataOffset]; // Set drawing colour, Get first data element.
       for (n=DataOffset; n<DataWindow+DataOffset+1; n++) {
           data1 = data2; data2 = InputBuffer1[n+1]; Y=40.0;
           for (scan=0; scan<4; scan++) {
               if (scan == 2) { Y=Y+40.0; } // Avoid center line on grid display
               if (Mask[scan] & data1) {
                  MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+HorzScale,Y),B_SOLID_HIGH);
                  if (!(Mask[scan] & data2)) {
                     StrokeLine(BPoint(X+HorzScale,Y+40.0),B_SOLID_HIGH); } }
               else {
                  MovePenTo(BPoint(X,Y+40.0)); StrokeLine(BPoint(X+HorzScale,Y+40.0),B_SOLID_HIGH);
                  if (Mask[scan] & data2) {
                     StrokeLine(BPoint(X+HorzScale,Y),B_SOLID_HIGH); } }
               Y=Y+80.0; }
           X=X+HorzScale; }}

    if (InputType == 2) { // 6 Direct digital inputs, 1 bit each for 1 byte from the parallel port.
       X=0.0; Y=60.0; SetHighColor(zeroColor); // Set drawing colour
       MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+60.0; MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+60.0;
       MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+80.0; MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+60.0;
       MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+60.0; MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH);
       SetHighColor(digitalColor); data2 = InputBuffer1[DataOffset]; // Set drawing colour, Get first data element.
       for (n=DataOffset; n<DataWindow+DataOffset+1; n++) {
           data1 = data2; data2 = InputBuffer1[n+1]; Y=20.0;
           for (scan=0; scan<6; scan++) {
               if (scan == 3) { Y=Y+20.0; } // Avoid center line on grid display
               if (Mask[scan] & data1) {
                  MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+HorzScale,Y),B_SOLID_HIGH);
                  if (!(Mask[scan] & data2)) {
                     StrokeLine(BPoint(X+HorzScale,Y+40.0),B_SOLID_HIGH); } }
               else {
                  MovePenTo(BPoint(X,Y+40.0)); StrokeLine(BPoint(X+HorzScale,Y+40.0),B_SOLID_HIGH);
                  if (Mask[scan] & data2) {
                     StrokeLine(BPoint(X+HorzScale,Y),B_SOLID_HIGH); } }
               Y=Y+60.0; }
           X=X+HorzScale; }}

    if (InputType == 3) { // 8 Direct digital inputs, 1 bit each for 1 byte from the parallel port.
       X=0.0; Y=60.0; SetHighColor(zeroColor); // Set drawing colour
       MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+40.0;
       MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+40.0;
       MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+40.0;
       MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+60.0;
       MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+40.0;
       MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+40.0;
       MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH); Y=Y+40.0;
       MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+800.0,Y),B_SOLID_HIGH);
       SetHighColor(digitalColor); data2 = InputBuffer1[DataOffset]; // Set drawing colour, Get first data element.
       for (n=DataOffset; n<DataWindow+DataOffset+1; n++) {
           data1 = data2; data2 = InputBuffer1[n+1]; Y=40.0;
           for (scan=0; scan<8; scan++) {
               if (scan == 4) { Y=Y+20.0; } // Avoid center line on grid display
               if (Mask[scan] & data1) {
                  MovePenTo(BPoint(X,Y)); StrokeLine(BPoint(X+HorzScale,Y),B_SOLID_HIGH);
                  if (!(Mask[scan] & data2)) {
                     StrokeLine(BPoint(X+HorzScale,Y+20.0),B_SOLID_HIGH); } }
               else {
                  MovePenTo(BPoint(X,Y+20.0)); StrokeLine(BPoint(X+HorzScale,Y+20.0),B_SOLID_HIGH);
                  if (Mask[scan] & data2) {
                     StrokeLine(BPoint(X+HorzScale,Y),B_SOLID_HIGH); } }
               Y=Y+40.0; }
           X=X+HorzScale; }}

    if (InputType == 4) { // Read 8 LED Bar ADC in, 9 possible values from the parallel port
       X=0.0; SetHighColor(analog1Color); // Set pen to analog input 1.
       data1 = 255 - InputBuffer1[DataOffset]; // Invert bit pattern
       if (data1 == 0)   { data = 0;   } // pattern 00000000 00000000
       if (data1 == 1)   { data = 32;  } // pattern 00000001 00000001
       if (data1 == 2)   { data = 64;  } // pattern 00000011 00000010
       if (data1 == 4)   { data = 96;  } // pattern 00000111 00000100
       if (data1 == 8)   { data = 128; } // pattern 00001111 00001000
       if (data1 == 16)  { data = 160; } // pattern 00011111 00010000
       if (data1 == 32)  { data = 192; } // pattern 00111111 00100000
       if (data1 == 64)  { data = 224; } // pattern 01111111 01000000
       if (data1 == 128) { data = 255; } // pattern 11111111 10000000
       MovePenTo(BPoint(X,((float)data-128.0)*scaling1+offset1));
       for (n=DataOffset+1; n<DataWindow+DataOffset+1; n++) {
           data1 = 255 - InputBuffer1[n]; // Invert bit pattern
           if (data1 == 0)   { data = 0;   } // pattern 00000000 00000000
           if (data1 == 1)   { data = 32;  } // pattern 00000001 00000001
           if (data1 == 2)   { data = 64;  } // pattern 00000011 00000010
           if (data1 == 4)   { data = 96;  } // pattern 00000111 00000100
           if (data1 == 8)   { data = 128; } // pattern 00001111 00001000
           if (data1 == 16)  { data = 160; } // pattern 00011111 00010000
           if (data1 == 32)  { data = 192; } // pattern 00111111 00100000
           if (data1 == 64)  { data = 224; } // pattern 01111111 01000000
           if (data1 == 128) { data = 255; } // pattern 11111111 10000000
           X=X+HorzScale; StrokeLine(BPoint(X,((float)data-128.0)*scaling1+offset1), B_SOLID_HIGH); } }

    if (InputType == 10) { // Read 10 LEDs Bar ADC in, 9 possible values from the parallel port
       X=0.0; SetHighColor(analog1Color); // Set pen to analog input 1.
       data1 = InputBuffer1[DataOffset]; data2 = InputBuffer2[DataOffset] & 24; Display = 1023 - ((int)data1 + ((int)data2 * 32));
       if (Display == 0)   { data = 0;   } // pattern 0000000000 0000000000
       if (Display == 1)   { data = 26;  } // pattern 0000000001 0000000001  25.6
       if (Display == 2)   { data = 51;  } // pattern 0000000011 0000000010  51.2
       if (Display == 4)   { data = 78;  } // pattern 0000000111 0000000100  77.8
       if (Display == 8)   { data = 102; } // pattern 0000001111 0000001000 102.4
       if (Display == 16)  { data = 128; } // pattern 0000011111 0000010000 128.0
       if (Display == 32)  { data = 154; } // pattern 0000111111 0000100000 153.6
       if (Display == 64)  { data = 179; } // pattern 0001111111 0001000000 179.2
       if (Display == 128) { data = 204; } // pattern 0011111111 0010000000 204.8
       if (Display == 256) { data = 230; } // pattern 0111111111 0100000000 230.4
       if (Display == 512) { data = 255; } // pattern 1111111111 1000000000 256.0
       MovePenTo(BPoint(X,((float)data-128.0)*scaling1+offset1));
       for (n=DataOffset+1; n<DataWindow+DataOffset+1; n++) {
           data1 = InputBuffer1[n]; data2 = InputBuffer2[n] & 24; Display = 1023 - ((int)data1 + ((int)data2 * 32));
           if (Display == 0)   { data = 0;   } // pattern 0000000000 0000000000
           if (Display == 1)   { data = 26;  } // pattern 0000000001 0000000001  25.6
           if (Display == 2)   { data = 51;  } // pattern 0000000011 0000000010  51.2
           if (Display == 4)   { data = 78;  } // pattern 0000000111 0000000100  77.8
           if (Display == 8)   { data = 102; } // pattern 0000001111 0000001000 102.4
           if (Display == 16)  { data = 128; } // pattern 0000011111 0000010000 128.0
           if (Display == 32)  { data = 154; } // pattern 0000111111 0000100000 153.6
           if (Display == 64)  { data = 179; } // pattern 0001111111 0001000000 179.2
           if (Display == 128) { data = 204; } // pattern 0011111111 0010000000 204.8
           if (Display == 256) { data = 230; } // pattern 0111111111 0100000000 230.4
           if (Display == 512) { data = 255; } // pattern 1111111111 1000000000 256.0
           X=X+HorzScale; StrokeLine(BPoint(X,((float)data-128.0)*scaling1+offset1), B_SOLID_HIGH); }; Display = 5; }

    if (InputType == 5) { // 2 analog inputs, 4 bits each
       X=0.0; SetHighColor(analog1Color); data1=17*(InputBuffer1[DataOffset]&15); MovePenTo(BPoint(X,((float)data1-128.0)*scaling1+offset1)); // Set pen to analog input 1.
       for (n=DataOffset+1; n<DataWindow+DataOffset+1; n++) { data1=17*(InputBuffer1[n]&15); X=X+HorzScale; StrokeLine(BPoint(X,((float)data1-128.0)*scaling1+offset1), B_SOLID_HIGH); }
       X=0.0; SetHighColor(analog2Color); data2=17*(InputBuffer1[DataOffset]/16); MovePenTo(BPoint(X,((float)data2-128.0)*scaling2+offset2)); // Set pen to analog input 2.
       for (n=DataOffset+1; n<DataWindow+DataOffset+1; n++) { data2=17*(InputBuffer1[n]/16); X=X+HorzScale; StrokeLine(BPoint(X,((float)data2-128.0)*scaling2+offset2), B_SOLID_HIGH); } }
           
    if ((InputType > 5) && (InputType < 10)) { // 1 analog input 1 byte each.
       X=0.0; SetHighColor(analog1Color); MovePenTo(BPoint(X,((float)InputBuffer1[DataOffset]-128.0)*scaling1+offset1)); // Set pen to analog input 1.
       for (n=DataOffset+1; n<DataWindow+DataOffset+1; n++) { X=X+HorzScale; StrokeLine(BPoint(X,((float)InputBuffer1[n]-128.0)*scaling1+offset1),B_SOLID_HIGH); } } // Plot signal 1.

    if ((InputType > 7) && (InputType < 10)) { // 2 analog inputs 1 byte each.
       X=0.0; SetHighColor(analog2Color); MovePenTo(BPoint(X,((float)InputBuffer2[DataOffset]-128.0)*scaling2+offset2)); // Set pen to analog input 2.
       for (n=DataOffset+1;n<DataWindow+DataOffset+1;n++) { X=X+HorzScale; StrokeLine(BPoint(X,((float)InputBuffer2[n]-128.0)*scaling2+offset2),B_SOLID_HIGH); } } // Plot signal 2.

    if (InputType == 9) { // 4 analog inputs 1 byte each.
       X=0.0; SetHighColor(analog3Color); MovePenTo(BPoint(X,((float)InputBuffer3[DataOffset]-128.0)*scaling3+offset3)); // Set pen to analog input 3.
       for (n=DataOffset+1;n<DataWindow+DataOffset+1;n++) { X=X+HorzScale; StrokeLine(BPoint(X,((float)InputBuffer3[n]-128.0)*scaling3+offset3),B_SOLID_HIGH); } // Plot signal 3.
       X=0.0; SetHighColor(analog4Color); MovePenTo(BPoint(X,((float)InputBuffer4[DataOffset]-128.0)*scaling4+offset4)); // Set pen to analog input 4.
       for (n=DataOffset+1; n<DataWindow+DataOffset+1; n++) { X=X+HorzScale; StrokeLine(BPoint(X,((float)InputBuffer4[n]-128.0)*scaling4+offset4),B_SOLID_HIGH); } } } } // Plot signal 4.

// Digital_Colour_View.cpp
Digital_Colour_View::Digital_Colour_View(BRect rect, char *name) // Define 'Digital_Colour_View' constructor call.
             : BView(rect, name, B_FOLLOW_ALL, B_WILL_DRAW) { } ;

void Digital_Colour_View::Draw(BRect updateRect) { // Draws in view when show is called. 
 SetFont(be_plain_font); SetFontSize(12);          // Set font outline to plain. Set font size.
 X=0.0; Y=64.0; MovePenTo(BPoint(X,Y)); SetHighColor(blackColor); // Draw outline of display area. Set pen to Black.
 if (color == 1) { aColor->SetValue(gridColor);    DrawString("Set Grid Colour."); }
 if (color == 2) { aColor->SetValue(bulleyeColor); DrawString("Set Bull's Eye Colour."); }
 if (color == 3) { aColor->SetValue(digitalColor); DrawString("Set Digital plotting Colour."); }
 if (color == 4) { aColor->SetValue(zeroColor);    DrawString("Set Digital baseline Colour."); }
 if (color == 5) { aColor->SetValue(analog1Color); DrawString("Set Analog Signal 1 Colour."); }
 if (color == 6) { aColor->SetValue(analog2Color); DrawString("Set Analog Signal 2 Colour."); }
 if (color == 7) { aColor->SetValue(analog3Color); DrawString("Set Analog Signal 3 Colour."); }
 if (color == 8) { aColor->SetValue(analog4Color); DrawString("Set Analog Signal 4 Colour."); }
 if (color == 9) { aColor->SetValue(displayColor); DrawString("Set Display BackGround Colour."); }
 } // Label Window.

// Digital_Scope_Window.cpp
// BWindow(BRect frame, const char *title, window_type type, uint32 flags, uint32 workspaces = B_CURRENT_WORKSPACE) 
// BWindow(BRect frame, const char *title, window_look look, window_feel feel, uint32 flags, uint32 workspace = B_CURRENT_WORKSPACE) 
Digital_Scope_Window::Digital_Scope_Window(BRect frame) // Define 'Digital_Scope_Window' constructor call.
               : BWindow(frame,                     // Frame area of the window
                 "Digital/Analog Scope 0.91",       // Name of the window, appears in the Tab on top of the window.
                 B_TITLED_WINDOW, // Type of window (Type) B_UNTYPED_WINDOW, B_MODAL_WINDOW,B_BORDERED_WINDOW,B_TITLED_WINDOW,B_DOCUMENT_WINDOW,B_FLOATING_WINDOW
                 // Type of window (Look) B_DOCUMENT_WINDOW_LOOK, B_TITLED_WINDOW_LOOK, B_FLOATING_WINDOW_LOOK, B_MODAL_WINDOW_LOOK, B_BORDERED_WINDOW_LOOK, B_NO_BORDER_WINDOW_LOOK
                 // Type of window (Feel) B_NORMAL_WINDOW_FEEL, B_MODAL_SUBSET_WINDOW_FEEL, B_MODAL_APP_WINDOW_FEEL, B_MODAL_ALL_WINDOW_FEEL, B_FLOATING_SUBSET_WINDOW_FEEL, B_FLOATING_APP_WINDOW_FEEL, B_FLOATING_ALL_WINDOW_FEEL
                 B_NOT_RESIZABLE|B_NOT_ZOOMABLE|B_ASYNCHRONOUS_CONTROLS, // Flags. B_NOT_MOVABLE B_NOT_CLOSABLE, B_NOT_ZOOMABLE, B_NOT_MINIMIZABLE, B_NOT_H_RESIZABLE, B_NOT_V_RESIZABLE,
                                                                         // B_NOT_RESIZABLE, B_OUTLINE_RESIZE, B_WILL_ACCEPT_FIRST_CLICK, B_AVOID_FRONT, B_AVOID_FOCUS, B_NO_WORKSPACE_ACTIVATION,
                                                                         // B_NOT_ANCHORED_ON_ACTIVATE, B_ASYNCHRONOUS_CONTROLS, B_QUIT_ON_WINDOW_CLOSE
                 B_CURRENT_WORKSPACE ) { // Workspace displayed in. B_CURRENT_WORKSPACE or B_ALL_WORKSPACES or numbers of workspaces ORed together.

 X=420.0; Y=440.0; Z=200.0; // Position and width of sliders 
 slider1 = new BSlider(BRect(X,Y,X+Z,Y+30),"slider1","Scaling input 1",new BMessage(SLIDER_1),1,100); slider1->SetValue(scaling1*100); slider1->SetModificationMessage(new BMessage(SLIDER_1)); AddChild(slider1); Y=Y+38.0;
 slider2 = new BSlider(BRect(X,Y,X+Z,Y+40),"slider2","Scaling input 2",new BMessage(SLIDER_2),1,100); slider2->SetValue(scaling2*100); slider2->SetModificationMessage(new BMessage(SLIDER_2)); AddChild(slider2); Y=Y+38.0;
 slider3 = new BSlider(BRect(X,Y,X+Z,Y+30),"slider3","Scaling input 3",new BMessage(SLIDER_3),1,100); slider3->SetValue(scaling3*100); slider3->SetModificationMessage(new BMessage(SLIDER_3)); AddChild(slider3); Y=Y+38.0;
 slider4 = new BSlider(BRect(X,Y,X+Z,Y+30),"slider4","Scaling input 4",new BMessage(SLIDER_4),1,100); slider4->SetValue(scaling4*100); slider4->SetModificationMessage(new BMessage(SLIDER_4)); AddChild(slider4);
 X=X+Z+10.0; Y=440.0;
 slider5 = new BSlider(BRect(X,Y,X+Z,Y+30),"slider5","Offset input 1",new BMessage(SLIDER_5),0,399); slider5->SetValue(offset1); slider5->SetModificationMessage(new BMessage(SLIDER_5)); AddChild(slider5); Y=Y+38.0;
 slider6 = new BSlider(BRect(X,Y,X+Z,Y+30),"slider6","Offset input 2",new BMessage(SLIDER_6),0,399); slider6->SetValue(offset2); slider6->SetModificationMessage(new BMessage(SLIDER_6)); AddChild(slider6); Y=Y+38.0;
 slider7 = new BSlider(BRect(X,Y,X+Z,Y+30),"slider7","Offset input 3",new BMessage(SLIDER_7),0,399); slider7->SetValue(offset3); slider7->SetModificationMessage(new BMessage(SLIDER_7)); AddChild(slider7); Y=Y+38.0;
 slider8 = new BSlider(BRect(X,Y,X+Z,Y+30),"slider8","Offset input 4",new BMessage(SLIDER_8),0,399); slider8->SetValue(offset4); slider8->SetModificationMessage(new BMessage(SLIDER_8)); AddChild(slider8);

 X=20.0; Y=470.0; Z = 200.0;
 slider9 = new BSlider(BRect(X,Y,X+Z,Y+30),"slider9","Horizontal Magnify",      new BMessage(SLIDER_9),1,10); slider9->SetModificationMessage(new BMessage(SLIDER_9));
 slider9->SetValue(HorzScale); slider9->SetHashMarks(B_HASH_MARKS_BOTH); slider9->SetHashMarkCount(10); AddChild(slider9); Y=Y+35.0;
 slider0 = new BSlider(BRect(X,Y,X+Z,Y+30),"slider0","Horizontal Rough Offset", new BMessage(SLIDER_0),0,31); slider0->SetModificationMessage(new BMessage(SLIDER_0));
 slider0->SetValue(HorzOffset1); slider0->SetHashMarks(B_HASH_MARKS_BOTH); slider0->SetHashMarkCount(16); AddChild(slider0); Y=Y+35.0;
 sliderA = new BSlider(BRect(X,Y,X+Z,Y+30),"sliderA","Horizontal Fine Offset",  new BMessage(SLIDER_A),0,31); sliderA->SetModificationMessage(new BMessage(SLIDER_A));
 sliderA->SetValue(HorzOffset2); sliderA->SetHashMarks(B_HASH_MARKS_BOTH); sliderA->SetHashMarkCount(16); AddChild(sliderA); Y=Y+35.0;
 HorzOffset0 = HorzOffset1 * 32 + HorzOffset2;

 X=220.0; Y=440.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button00","Sample in Data", new BMessage(MENU_SCAN_PLOT))); Y=Y+25.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button01","2 Analog Inputs",new BMessage(MENU_SCAN_2)));    Y=Y+25.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button02","Analog(8 LEDs)", new BMessage(MENU_BAR_ADC08))); Y=Y+25.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button03","2 Digital Bits", new BMessage(MENU_DIGIT2)));    Y=Y+25.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button04","6 Digital Bits", new BMessage(MENU_DIGIT6)));    Y=Y+25.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button05","Read Port Data", new BMessage(MENU_READ_PORT))); Y=Y+25.0;
 X=X+100.0; Y=440.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button06","1 Analog Input", new BMessage(MENU_SCAN_1)));    Y=Y+25.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button07","4 Analog Inputs",new BMessage(MENU_SCAN_4)));    Y=Y+25.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button08","Analog(10 LEDs)",new BMessage(MENU_BAR_ADC10))); Y=Y+25.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button09","4 Digital Bits", new BMessage(MENU_DIGIT4)));    Y=Y+25.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button10","8 Digital Bits", new BMessage(MENU_DIGIT8)));    Y=Y+25.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button11","2x4 Bits Analog",new BMessage(MENU_2_4_ADC)));   Y=Y+30.0;

 X=20.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button14","Grid Modes",      new BMessage(MENU_GRID_SWCTH))); X=X+100.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button15","Refresh Display", new BMessage(REFRESH_DISPLAY))); X=X+100.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button16","Offset Display",  new BMessage(MENU_OFFSET)));     X=X+100.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button17","Normal Scaling",  new BMessage(MENU_NORMAL)));     X=X+100.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button18","Fake Sampling",   new BMessage(MENU_FAKE_SCAN)));  X=X+100.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button19","Not Defined",     new BMessage(APP_SLEEP)));       X=X+100.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button20","Not Defined",     new BMessage(APP_SLEEP)));       X=X+100.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button21","Not Defined",     new BMessage(APP_SLEEP)));       X=X+100.0;
 X=20.0; Y=Y+25.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button23","Bull's Eye",      new BMessage(MENU_BULLS_EYE))); X=X+100.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button24","StandardDisplay", new BMessage(MENU_RESTORE)));   X=X+100.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button25","Center Display",  new BMessage(MENU_CENTER)));    X=X+100.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button26","Max Scaling",     new BMessage(MENU_MAX)));       X=X+100.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button27","Clear Sampling",  new BMessage(MENU_FILE_NEW)));  X=X+100.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button28","Not Defined",     new BMessage(APP_SLEEP)));      X=X+100.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button29","Not Defined",     new BMessage(APP_SLEEP)));      X=X+100.0;
 AddChild(new BButton(BRect(X,Y,X+95.0,Y+10.0),"Button30","Not Defined",     new BMessage(APP_SLEEP)));      X=X+100.0;

 BRect rMenu;                    // Set the Window Bpoints into the rectangle object(aRect) to define a MenuBar.
 BMenu *menu;                    // Main Menu
 BMenu *submenu;                 // Sub Menu
 rMenu = Bounds();               // Find active area of view.
 menubar = new BMenuBar(rMenu,   // Invoke a new dynamically allocated menubar object inside the window
          "Basic GUI Menu Bar"); // by calling the constructor. Plus supply a ID name for the menu.
 AddChild(menubar);              // Add menubar to window created.

 menu = new BMenu("File"); // Create File Bmenu. SetEnabled()
 menuitem = new BMenuItem("New",         new BMessage(MENU_FILE_NEW),'N');   menuitem->SetEnabled(true);  menu->AddItem(menuitem); // Clear all buffers. Not used.
 menuitem = new BMenuItem("Open",        new BMessage(MENU_FILE_OPEN),'O');  menuitem->SetEnabled(true);  menu->AddItem(menuitem); // Load a file into buffer. Not used.
 menuitem = new BMenuItem("Close",       new BMessage(MENU_FILE_CLOSE),'W'); menuitem->SetEnabled(false); menu->AddItem(menuitem); // Close loading a file into buffer. Not used.
 menuitem = new BMenuItem("Save",        new BMessage(MENU_FILE_SAVE),'S');  menuitem->SetEnabled(false); menu->AddItem(menuitem); // Save buffer into a file. Not used.
 menuitem = new BMenuItem("Save as",     new BMessage(MENU_FILE_SAVEAS));    menuitem->SetEnabled(true);  menu->AddItem(menuitem); // Save buffer into a file. Not used.
 menu->AddSeparatorItem();
 menuitem = new BMenuItem("Page Setup",  new BMessage(MENU_FILE_PAGESETUP)); menuitem->SetEnabled(false); menu->AddItem(menuitem); // Setup page margins. Not used.
 menuitem = new BMenuItem("Print",       new BMessage(MENU_FILE_PRINT),'P'); menuitem->SetEnabled(false); menu->AddItem(menuitem); // Print data. Not used.
 menu->AddSeparatorItem();
 submenu = new BMenu("Preferences"); // Preferences for program. Not used.
    menuitem = new BMenuItem("Set BackGround Colour", new BMessage(MENU_FILE_PREFS_DISPLAY));  menuitem->SetEnabled(true); submenu->AddItem(menuitem); // Preferences for program.
    menuitem = new BMenuItem("Set Grid Colour",       new BMessage(MENU_FILE_PREFS_GRID));     menuitem->SetEnabled(true); submenu->AddItem(menuitem); // Preferences for program.
    menuitem = new BMenuItem("Set Bull Eye Colour",   new BMessage(MENU_FILE_PREFS_BULLsEYE)); menuitem->SetEnabled(true); submenu->AddItem(menuitem); // Preferences for program.
    submenu->AddSeparatorItem();
    menuitem = new BMenuItem("Set Digital Colour",    new BMessage(MENU_FILE_PREFS_DIGITAL));  menuitem->SetEnabled(true); submenu->AddItem(menuitem); // Preferences for program.
    menuitem = new BMenuItem("Set Zero Line Colour",  new BMessage(MENU_FILE_PREFS_ZERO));     menuitem->SetEnabled(true); submenu->AddItem(menuitem); // Preferences for program.
    submenu->AddSeparatorItem();
    menuitem = new BMenuItem("Set Analog 1 Colour",   new BMessage(MENU_FILE_PREFS_ANALOG1));  menuitem->SetEnabled(true); submenu->AddItem(menuitem); // Preferences for program.
    menuitem = new BMenuItem("Set Analog 2 Colour",   new BMessage(MENU_FILE_PREFS_ANALOG2));  menuitem->SetEnabled(true); submenu->AddItem(menuitem); // Preferences for program.
    menuitem = new BMenuItem("Set Analog 3 Colour",   new BMessage(MENU_FILE_PREFS_ANALOG3));  menuitem->SetEnabled(true); submenu->AddItem(menuitem); // Preferences for program.
    menuitem = new BMenuItem("Set Analog 4 Colour",   new BMessage(MENU_FILE_PREFS_ANALOG4));  menuitem->SetEnabled(true); submenu->AddItem(menuitem); // Preferences for program.
    menu->AddItem(submenu);
 menu->AddSeparatorItem();
 menuitem = new BMenuItem("About",       new BMessage(MENU_FILE_ABOUT));     menuitem->SetEnabled(true);  menu->AddItem(menuitem); // About this program.
 menu->AddSeparatorItem();
 menuitem = new BMenuItem("Quit",        new BMessage(MENU_FILE_QUIT),'Q');  menuitem->SetEnabled(true);  menu->AddItem(menuitem); // Exit program.
 menubar->AddItem(menu); // Adds File Bmenu to menu bar.

 menu = new BMenu("Edit"); // Create Edit Bmenu.
 menuitem = new BMenuItem("Undo",       new BMessage(MENU_EDIT_UNDO),'Z');    menuitem->SetEnabled(false); menu->AddItem(menuitem); // Reverse last changes. Not used.
 menu->AddSeparatorItem();
 menuitem = new BMenuItem("Cut",        new BMessage(MENU_EDIT_CUT),'X');     menuitem->SetEnabled(false); menu->AddItem(menuitem); // Move selected data to clipboard. Not used.
 menuitem = new BMenuItem("Copy",       new BMessage(MENU_EDIT_COPY),'C');    menuitem->SetEnabled(false); menu->AddItem(menuitem); // Copy selected data to clipboard. Not used.
 menuitem = new BMenuItem("Paste",      new BMessage(MENU_EDIT_PASTE),'V');   menuitem->SetEnabled(false); menu->AddItem(menuitem); // Paste clipboard to present position. Not used.
 menuitem = new BMenuItem("Select All", new BMessage(MENU_EDIT_SELECT),'A');  menuitem->SetEnabled(false); menu->AddItem(menuitem); // Select all data. Not used.
 menu->AddSeparatorItem();
 menuitem = new BMenuItem("Find",       new BMessage(MENU_EDIT_FIND),'F');    menuitem->SetEnabled(false); menu->AddItem(menuitem); // Find data sequence. Not used.
 menuitem = new BMenuItem("Replace",    new BMessage(MENU_EDIT_REPLACE),'='); menuitem->SetEnabled(false); menu->AddItem(menuitem); // Replace data sequence. Not used.
 menubar->AddItem(menu); // Adds Edit Bmenu to menu bar.
 
 menu = new BMenu("Port Testing"); // Create Test Bmenu.
 menu->AddItem(new BMenuItem("Count Up",          new BMessage(MENU_COUNT_UP),'0'));       // Count up parallel output
 menu->AddItem(new BMenuItem("Count Down",        new BMessage(MENU_COUNT_DOWN),'1'));     // Count down parallel output
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("Read Port values",  new BMessage(MENU_READ_PORT),'2'));      // Reads parallel port's registers
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("Write   0 to Port", new BMessage(MENU_WRITE_PORT_0),'3'));   // Write 0 to Port
 menu->AddItem(new BMenuItem("Write  85 to Port", new BMessage(MENU_WRITE_PORT_85),'4'));  // Write 85 to Port
 menu->AddItem(new BMenuItem("Write 170 to Port", new BMessage(MENU_WRITE_PORT_170),'5')); // Write 170 to Port
 menu->AddItem(new BMenuItem("Write 255 to Port", new BMessage(MENU_WRITE_PORT_255),'6')); // Write 255 to Port
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("Sample and Plot",   new BMessage(MENU_SCAN_PLOT),'7')); // Read port then plot
 menu->AddItem(new BMenuItem("Fake Sampling Data",new BMessage(MENU_FAKE_SCAN),'8')); // Fake a scan.
 menu->AddItem(new BMenuItem("Restore Settings",  new BMessage(MENU_RESTORE),'9'));   // Set controls to defaults.
 menu->SetRadioMode(true); // Puts check mark beside selected function.
 menubar->AddItem(menu); // Adds Project Bmenu to menu bar.

 menu = new BMenu("Set Sampling Timing"); // Create Timing Bmenu.
 menuitem = new BMenuItem("1 sample per second",       new BMessage(MENU_TIMING_1)); if (Timing == 1000000) { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // 1 samples. per second
 menuitem = new BMenuItem("10 samples per second",     new BMessage(MENU_TIMING_2)); if (Timing == 100000)  { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // 10 samples per second
 menuitem = new BMenuItem("100 samples per second",    new BMessage(MENU_TIMING_3)); if (Timing == 10000)   { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // 100 samples per second
 menuitem = new BMenuItem("1000 samples per second",   new BMessage(MENU_TIMING_4)); if (Timing == 1000)    { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // 1000 samples per second
 menuitem = new BMenuItem("10000 samples per second",  new BMessage(MENU_TIMING_5)); if (Timing == 100)     { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // 10000 samples per second
 menuitem = new BMenuItem("100000 samples per second", new BMessage(MENU_TIMING_6)); if (Timing == 10)      { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // 100000 samples per second
 menuitem = new BMenuItem("MAX samples per second",    new BMessage(MENU_TIMING_7)); if (Timing == 0)       { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // MAX samples per second
 menu->SetRadioMode(true); // Puts check mark beside selected speed.
 menubar->AddItem(menu); // Adds Timing Bmenu to menu bar.

 menu = new BMenu("Sample Type"); // Create Sample Type Bmenu.
 menuitem = new BMenuItem("  Two Digital Inputs",   new BMessage(MENU_DIGIT2)); if (InputType == 0) { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // Read 8 digital lines in.
 menuitem = new BMenuItem(" Four Digital Inputs",   new BMessage(MENU_DIGIT4)); if (InputType == 1) { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // Read 8 digital lines in.
 menuitem = new BMenuItem("  Six Digital Inputs",   new BMessage(MENU_DIGIT6)); if (InputType == 2) { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // Read 8 digital lines in.
 menuitem = new BMenuItem("Eight Digital Inputs",   new BMessage(MENU_DIGIT8)); if (InputType == 3) { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // Read 8 digital lines in.
 menu->AddSeparatorItem();
 menuitem = new BMenuItem("Read 8 LEDs Bar ADC in.",      new BMessage(MENU_BAR_ADC08)); if (InputType == 4)  { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // Read 8 LEDs Bar ADC in.
 menuitem = new BMenuItem("Read 10 LEDs Bar ADC in.",     new BMessage(MENU_BAR_ADC10)); if (InputType == 10) { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // Read 10 LEDs Bar ADC in.
 menuitem = new BMenuItem("Dual 4-Bit Flash ADC Inputs",  new BMessage(MENU_2_4_ADC)); if (InputType == 5) { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // Read 2 Flash 4-Bit ADCs.
 menuitem = new BMenuItem("Single 8-Bit Flash ADC Input", new BMessage(MENU_1_8_ADC)); if (InputType == 6) { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // Read 1 Flash 8-Bit ADC.
 menu->AddSeparatorItem();
 menuitem = new BMenuItem("Scanning Single Analog input", new BMessage(MENU_SCAN_1));  if (InputType == 7) { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // Scan for one 8 bit value.
 menuitem = new BMenuItem("Scanning Dual Analog input",   new BMessage(MENU_SCAN_2));  if (InputType == 8) { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // Scan for two 8 bit values.
 menuitem = new BMenuItem("Scanning Quad Analog input",   new BMessage(MENU_SCAN_4));  if (InputType == 9) { menuitem->SetMarked(true); } ; menu->AddItem(menuitem); // Scan for four 8 bit values.
 menu->SetRadioMode(true); // Puts check mark beside selected function.
 menubar->AddItem(menu); // Adds Sample Type Bmenu to menu bar.

// menu = new BMenu("Trigger Source"); // Create Source Bmenu.
// menu->AddItem(new BMenuItem("Input 0", new BMessage(MENU_TRIGGER_0))); // Input 0 source of triggering scan.
// menu->AddItem(new BMenuItem("Input 1", new BMessage(MENU_TRIGGER_1))); // Input 1 source of triggering scan.
// menu->AddItem(new BMenuItem("Input 2", new BMessage(MENU_TRIGGER_2))); // Input 2 source of triggering scan.
// menu->AddItem(new BMenuItem("Input 3", new BMessage(MENU_TRIGGER_3))); // Input 3 source of triggering scan.
// menu->AddItem(new BMenuItem("Input 4", new BMessage(MENU_TRIGGER_4))); // Input 4 source of triggering scan.
// menu->AddItem(new BMenuItem("Input 5", new BMessage(MENU_TRIGGER_5))); // Input 5 source of triggering scan.
// menu->AddItem(new BMenuItem("Input 6", new BMessage(MENU_TRIGGER_6))); // Input 6 source of triggering scan.
// menu->AddItem(new BMenuItem("Input 7", new BMessage(MENU_TRIGGER_7))); // Input 7 source of triggering scan.
// menu->SetRadioMode(true); // Puts check mark beside selected function.
// menubar->AddItem(menu); // Adds Source Bmenu to menu bar.

 menu = new BMenu("Triggering Types"); // Create Trigger Bmenu.
 menuitem = new BMenuItem("Single Shot on Trigger", new BMessage(MENU_SINGLE_SHOT));   if (ScanMode == 1) { menuitem->SetMarked(true); } ; menuitem->SetEnabled(false); menu->AddItem(menuitem); // Run only once when trigger conditions meet.
 menuitem = new BMenuItem("Run on every Trigger",   new BMessage(MENU_EVERY_TRIGGER)); if (ScanMode == 2) { menuitem->SetMarked(true); } ; menuitem->SetEnabled(false); menu->AddItem(menuitem); // Run everytime when trigger conditions meet.
 menu->SetRadioMode(true); // Puts check mark beside selected function.
 menubar->AddItem(menu); // Adds Trigger Bmenu to menu bar.

 menu = new BMenu("Triggering Modes"); // Create Trigger Bmenu.
 menuitem = new BMenuItem("Trigger not needed",      new BMessage(MENU_NO_TRIGGER));      if (TriggerMode == 0) { menuitem->SetMarked(true); } ; menuitem->SetEnabled(false); menu->AddItem(menuitem);  // Just scan.
 menuitem = new BMenuItem("Trigger on rise",         new BMessage(MENU_RISING_TRIGGER));  if (TriggerMode == 1) { menuitem->SetMarked(true); } ; menuitem->SetEnabled(false); menu->AddItem(menuitem);  // Trigger scan if signal rises.
 menuitem = new BMenuItem("Trigger on fall",         new BMessage(MENU_FALLING_TRIGGER)); if (TriggerMode == 2) { menuitem->SetMarked(true); } ; menuitem->SetEnabled(false); menu->AddItem(menuitem);  // Trigger scan if signal falls.
 menuitem = new BMenuItem("Trigger on mid crossing", new BMessage(MENU_MID_TRIGGER));     if (TriggerMode == 3) { menuitem->SetMarked(true); } ; menuitem->SetEnabled(false); menu->AddItem(menuitem);  // Trigger scan if signal cross mid value.
 menuitem = new BMenuItem("Trigger on value",        new BMessage(MENU_MID_TRIGGER));     if (TriggerMode == 4) { menuitem->SetMarked(true); } ; menuitem->SetEnabled(false); menu->AddItem(menuitem);  // Trigger scan if signal cross mid value.
 menu->SetRadioMode(true); // Puts check mark beside selected function.
 menubar->AddItem(menu); // Adds Trigger Bmenu to menu bar.

 menu = new BMenu("Pulse Timing"); // Create Pulse Bmenu.
 menu->AddItem(new BMenuItem("Pulse Off",                new BMessage(MENU_PULSE_OFF))); // Pulse off.
 menu->AddItem(new BMenuItem("Pulse every .1 seconds.",  new BMessage(MENU_PULSE_0)));   // Pulse every .1 seconds.
 menu->AddItem(new BMenuItem("Pulse every 1 seconds.",   new BMessage(MENU_PULSE_1)));   // Pulse every second.
 menu->AddItem(new BMenuItem("Pulse every 10 seconds.",  new BMessage(MENU_PULSE_10)));  // Pulse every 10 seconds.
 menu->AddItem(new BMenuItem("Pulse every 100 seconds.", new BMessage(MENU_PULSE_100))); // Pulse every 100 seconds.
 menubar->AddItem(menu); // Adds Pulse Bmenu to menu bar.

 menu = new BMenu("DeBug Levels"); // Create DeBug Bmenu.
 menu->AddItem(new BMenuItem("DeBug Off",          new BMessage(MENU_DEBUG_OFF))); // DeBug off.
 menu->AddItem(new BMenuItem("DeBug low level",    new BMessage(MENU_DEBUG_1)));   // DeBug low level only.
 menu->AddItem(new BMenuItem("DeBug medium level", new BMessage(MENU_DEBUG_2)));   // DeBug to medium levels.
 menu->AddItem(new BMenuItem("DeBug all",          new BMessage(MENU_DEBUG_3)));   // DeBug all levels.
 menu->SetRadioMode(true); menubar->AddItem(menu); Show(); } // Puts check mark beside selected function. Adds DeBug Bmenu to menu bar.

bool Digital_Scope_Window::QuitRequested() { // ::QuitRequested. Close button on window pressed. A Blooper call to Bwindows. 
 be_app->PostMessage(B_QUIT_REQUESTED); return(true); } ; // Send a message to the global variable 'be_app' to tell the program to quit. Tell the system (Blooper) to close window.

// Digital_Scope_Window::MessageReceived.  Called when a message is received by our application.
void Digital_Scope_Window::MessageReceived(BMessage *message) {
 if (DeBug == 2) { alert = new BAlert("DeBug","Digital_Scope_Window::MessageReceived().","OK"); result = alert->Go(); }

 switch(message->what) {
  case MENU_FILE_NEW: {  // Clear all data buffers.
       for (n=0; n<1028; n++) { InputBuffer1[n] = 128; InputBuffer2[n] = 128; InputBuffer3[n] = 128; InputBuffer4[n] = 128; }
       Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; }

// FilePanel(file_panel_mode mode = B_OPEN_PANEL,
//           BMessenger* target = NULL,
//           entry_ref *panel_directory = NULL,
//           uint32 node_flavors = 0,
//          B bool allow_multiple_selection = true,
//          B BMessage *message = NULL,
//           BRefFilter *filter = NULL,
//          B bool modal = false,
//           bool hide_when_done = true)
  case MENU_FILE_OPEN: { // Load a file into buffers.
       if (DeBug == 3) { alert = new BAlert("Info","Open and Load.","OK"); result = alert->Go(); }
       LoadFlag = true; SaveFlag = false; // Load file.
       if (OpenFilePanel == NULL) {// Create a File-Open panel if it does not exist.  Use defaults but no multi-file selection.
          OpenFilePanel = new BFilePanel(B_OPEN_PANEL,NULL,NULL,0,false,NULL,NULL,false,true); }
       OpenFilePanel->Show(); Display = 5; break; } // Display panel. Get file to load.

  case MENU_FILE_CLOSE: {
       alert = new BAlert("Info","Not needed in this program.","OK"); result = alert->Go(); break; } // Close loading a file into buffer.

  case MENU_FILE_SAVE: { // Save buffers into a file.
       if (DeBug == 3) { alert = new BAlert("Info","Open and Save.","OK"); result = alert->Go(); }
       LoadFlag = false; SaveFlag = true; // Save file.
       if (SaveFilePanel == NULL) { // Create a File-Save panel if it does not exist.
          SaveFilePanel = new BFilePanel(B_SAVE_PANEL,NULL,NULL,0,false,NULL,NULL,false,true); }
       SaveFilePanel->Show(); break; } // Display panel. Get file to save.
      
  case MENU_FILE_SAVEAS: { // Save buffers into a file.
       if (DeBug == 3) { alert = new BAlert("Info","Open and SaveAs.","OK"); result = alert->Go(); }
       LoadFlag = false; SaveFlag = true; // Save file.
       if (SaveFilePanel == NULL) { // Create a File-SaveAs panel if it does not exist.
          SaveFilePanel = new BFilePanel(B_SAVE_PANEL,NULL,NULL,0,false,NULL,NULL,false,true); }
       SaveFilePanel->Show(); break; } // Display panel. Get file to save.

  case MENU_FILE_PAGESETUP: { alert = new BAlert("Info","To be added.","OK"); result = alert->Go(); break; } // Setup page margins.
  case MENU_FILE_PRINT:     { alert = new BAlert("Info","To be added.","OK"); result = alert->Go(); break; } // Print data.

  case MENU_FILE_PREFS_GRID: { // Preferences to Set Grid Colour for program.
       color = 1; be_app->PostMessage(COLOUR_SET); aWindow2->Minimize(false); break; }
  case MENU_FILE_PREFS_BULLsEYE: { // Preferences to Set Bull Eye Colour for program.
       color = 2; be_app->PostMessage(COLOUR_SET); aWindow2->Minimize(false); break; }
  case MENU_FILE_PREFS_DIGITAL: { // Preferences to Set Digital Plotting Colour for program.
       color = 3; be_app->PostMessage(COLOUR_SET); aWindow2->Minimize(false); break; }
  case MENU_FILE_PREFS_ZERO: { // Preferences to Set Digital Plotting Zero Level Colour for program.
       color = 4; be_app->PostMessage(COLOUR_SET); aWindow2->Minimize(false); break; }
  case MENU_FILE_PREFS_ANALOG1: { // Preferences to Set Analog 1 Plotting Colour for program.
       color = 5; be_app->PostMessage(COLOUR_SET); aWindow2->Minimize(false); break; }
  case MENU_FILE_PREFS_ANALOG2: { // Preferences to Set Analog 2 Plotting Colour for program.
       color = 6; be_app->PostMessage(COLOUR_SET); aWindow2->Minimize(false); break; }
  case MENU_FILE_PREFS_ANALOG3: { // Preferences to Set Analog 3 Plotting Colour for program.
       color = 7; be_app->PostMessage(COLOUR_SET); aWindow2->Minimize(false); break; }
  case MENU_FILE_PREFS_ANALOG4: { // Preferences to Set Analog 4 Plotting Colour for program.
       color = 8; be_app->PostMessage(COLOUR_SET); aWindow2->Minimize(false); break; }
  case MENU_FILE_PREFS_DISPLAY: { // Preferences to Set Display BackGround Colour for program.
       color = 9; be_app->PostMessage(COLOUR_SET); aWindow2->Minimize(false); break; }

  case MENU_FILE_ABOUT: { be_app->PostMessage(B_ABOUT_REQUESTED); break; } // Displays alert. About the program.
  case MENU_FILE_QUIT:  { be_app->PostMessage(B_QUIT_REQUESTED);  break; } // Exit program, sends a message to BApplication.

  case MENU_EDIT_UNDO:    { alert = new BAlert("Info","Not needed in this program.","OK"); result = alert->Go(); break; } // Reverse last changes.
  case MENU_EDIT_CUT:     { alert = new BAlert("Info","Not needed in this program.","OK"); result = alert->Go(); break; } // Move selected data to clipboard.
  case MENU_EDIT_COPY:    { alert = new BAlert("Info","Not needed in this program.","OK"); result = alert->Go(); break; } // Copy selected data to clipboard.
  case MENU_EDIT_PASTE:   { alert = new BAlert("Info","Not needed in this program.","OK"); result = alert->Go(); break; } // Paste clipboard to present position.
  case MENU_EDIT_SELECT:  { alert = new BAlert("Info","Not needed in this program.","OK"); result = alert->Go(); break; } // Select all data.
  case MENU_EDIT_FIND:    { alert = new BAlert("Info","Not needed in this program.","OK"); result = alert->Go(); break; } // Find data sequence.
  case MENU_EDIT_REPLACE: { alert = new BAlert("Info","Not needed in this program.","OK"); result = alert->Go(); break; } // Replace data sequence.

  case MENU_COUNT_UP:   { be_app->PostMessage(APP_COUNT_UP);   break; } // Count up parallel output.
  case MENU_COUNT_DOWN: { be_app->PostMessage(APP_COUNT_DOWN); break; } // Count down parallel output
  case MENU_READ_PORT:  { be_app->PostMessage(APP_READ_PORT);  break; } // Read values off parallel port's inputs.

  case MENU_WRITE_PORT_0:   { data =   0; be_app->PostMessage(APP_WRITE_PORT); break; } // Write a 0 on parallel port's outputs.
  case MENU_WRITE_PORT_85:  { data =  85; be_app->PostMessage(APP_WRITE_PORT); break; } // Write a 85 on parallel port's outputs.
  case MENU_WRITE_PORT_170: { data = 170; be_app->PostMessage(APP_WRITE_PORT); break; } // Write a 170 on parallel port's outputs.
  case MENU_WRITE_PORT_255: { data = 255; be_app->PostMessage(APP_WRITE_PORT); break; } // Write a 255 on parallel port's outputs.

  case MENU_SCAN_PLOT: { be_app->PostMessage(APP_SCAN_PLOT); Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Collect data, display.

  case MENU_FAKE_SCAN: { // Fake sample data.
       for (n=0; n<1024; n++) { // Fill all data elements.
           InputBuffer1[n] = n & 255; // Binary counter in digital mode, sawtooth on analog 1 input.
           InputBuffer2[n] = (int)(128+(sin((float)n/19.0)*126.0)); // Sine wave on analog 2 input.
           InputBuffer3[n] = 7 * (n & 32);                          // Square wave in analog 3 input.
           InputBuffer4[n] = rand() ^ rand() ^ rand() & 255; }      // Random noise on analog 4 input.
       Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; }   // Turn on plotting.  Refresh Data Display.

  case MENU_RESTORE: { // Restore settings
       TriggerMode = 0; ScanMode = 1; Timing = 1000;
       scaling1 = .2; scaling2 = .2; scaling3 = .2; scaling4 = .2; offset1 = 80.0; offset2 = 160.0; offset3 = 240.0; offset4 = 320.0;
       slider1->SetValue(scaling1*100); slider2->SetValue(scaling2*100); slider3->SetValue(scaling3*100); slider4->SetValue(scaling4*100);
       slider5->SetValue(offset1);      slider6->SetValue(offset2);      slider7->SetValue(offset3);      slider8->SetValue(offset4);
       Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; }

  case MENU_GRID_SWCTH: { // Turn Grid on and off.  Turn on plotting.  Refresh Data Display.
       GridMode = (GridMode++) & 7; Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; }

  case MENU_BULLS_EYE: { // Turn Bull's Eye on and off.  Turn on plotting.  Refresh Data Display.
       if (BullsEye) { BullsEye = false; } else { BullsEye = true; } 
       Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; }

  case REFRESH_DISPLAY: { // Turn on plotting.  Refresh Data Display.
       Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; }

  case MENU_OFFSET:{ // Center all displays.
       offset1 = 80.0; offset2 = 160.0; offset3 = 240.0; offset4 = 320.0;
       slider5->SetValue(offset1); slider6->SetValue(offset2); slider7->SetValue(offset3); slider8->SetValue(offset4); // Update sliders.
       Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Turn on plotting.  Refresh Data Display.

  case MENU_CENTER:{ // Center all displays.
       offset1 = 200.0, offset2 = 200.0, offset3 = 200.0, offset4 = 200.0; // Move mid point (value = 128) to match center line.
       slider5->SetValue(offset1); slider6->SetValue(offset2); slider7->SetValue(offset3); slider8->SetValue(offset4); // Update sliders.
       Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Turn on plotting.  Refresh Data Display.

  case MENU_NORMAL: {
       scaling1 = .2; scaling2 = .2; scaling3 = .2; scaling4 = .2;
       slider1->SetValue(scaling1*100); slider2->SetValue(scaling2*100); slider3->SetValue(scaling3*100); slider4->SetValue(scaling4*100); // Update sliders.
       Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Turn on plotting.  Refresh Data Display.

  case MENU_MAX: {
       scaling1 = 1.5; scaling2 = 1.5; scaling3 = 1.5; scaling4 = 1.5;
       slider1->SetValue(scaling1*100); slider2->SetValue(scaling2*100); slider3->SetValue(scaling3*100); slider4->SetValue(scaling4*100); // Update sliders.
       Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Turn on plotting.  Refresh Data Display.

  case MENU_TIMING_1: { Timing = 1000000; break; } // 1 sample per second.
  case MENU_TIMING_2: { Timing = 100000; break; } // 10 samples per second.
  case MENU_TIMING_3: { Timing = 10000; break; } // 100 samples per second.
  case MENU_TIMING_4: { Timing = 1000; break; } // 1000 samples per second.
  case MENU_TIMING_5: { Timing = 100; break; } // 10000 samples per second.
  case MENU_TIMING_6: { Timing = 10; break; } // 100000 samples per second.
  case MENU_TIMING_7: { Timing = 0; break; } // MAX samples per second.

  case MENU_DIGIT2: {    InputType = 0;  Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Read 2 digital lines in.
  case MENU_DIGIT4: {    InputType = 1;  Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Read 4 digital lines in.
  case MENU_DIGIT6: {    InputType = 2;  Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Read 6 digital lines in.
  case MENU_DIGIT8: {    InputType = 3;  Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Read 8 digital lines in.
  case MENU_BAR_ADC08: { InputType = 4;  Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Read 8 LEDs Bar ADC in for 9 analog levels.
  case MENU_BAR_ADC10: { InputType = 10; Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Read 10 LEDs Bar ADC in for 11 analog levels.
  case MENU_2_4_ADC: {   InputType = 5;  Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Read 2 Flash 4-Bit ADCs.
  case MENU_1_8_ADC: {   InputType = 6;  Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Read 1 Flash 8-Bit ADC.
  case MENU_SCAN_1: {    InputType = 7;  Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Scan for one 8 bit analog value.
  case MENU_SCAN_2: {    InputType = 8;  Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Scan for two 8 bit analog values.
  case MENU_SCAN_4: {    InputType = 9;  Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Scan for four 8 bit analog values.

  case MENU_TRIGGER_0: { Source = 0; break; } // Input 0 source of triggering scan.
  case MENU_TRIGGER_1: { Source = 1; break; } // Input 1 source of triggering scan.
  case MENU_TRIGGER_2: { Source = 2; break; } // Input 2 source of triggering scan.
  case MENU_TRIGGER_3: { Source = 3; break; } // Input 3 source of triggering scan.
  case MENU_TRIGGER_4: { Source = 4; break; } // Input 4 source of triggering scan.
  case MENU_TRIGGER_5: { Source = 5; break; } // Input 5 source of triggering scan.
  case MENU_TRIGGER_6: { Source = 6; break; } // Input 6 source of triggering scan.
  case MENU_TRIGGER_7: { Source = 7; break; } // Input 7 source of triggering scan.

  case MENU_SINGLE_SHOT:   { ScanMode = 1; break; } // Run only once when trigger conditions meet.
  case MENU_EVERY_TRIGGER: { ScanMode = 2; break; } // Run everytime when trigger conditions meet.

  case MENU_NO_TRIGGER:      { TriggerMode = 0; break; } // Scans without waiting.
  case MENU_RISING_TRIGGER:  { TriggerMode = 1; break; } // Trigger scan if signal rises.
  case MENU_FALLING_TRIGGER: { TriggerMode = 2; break; } // Trigger scan if signal falls.
  case MENU_MID_TRIGGER:     { TriggerMode = 3; break; } // Trigger scan if signal cross mid value.
  case MENU_VALUE_TRIGGER:   { TriggerMode = 4; break; } // Trigger scan if signal equals value.

  case MENU_PULSE_OFF: { be_app->PostMessage(APP_PULSE_OFF); break; } // Sends a pulse message to BApplication to turn off Pulses.
  case MENU_PULSE_0:   { be_app->PostMessage(APP_PULSE_0);   break; } // Sends a pulse message to BApplication, every .1 seconds.
  case MENU_PULSE_1:   { be_app->PostMessage(APP_PULSE_1);   break; } // Sends a pulse message to BApplication, every 1 seconds.
  case MENU_PULSE_10:  { be_app->PostMessage(APP_PULSE_10);  break; } // Sends a pulse message to BApplication, every 10 seconds.
  case MENU_PULSE_100: { be_app->PostMessage(APP_PULSE_100); break; } // Sends a pulse message to BApplication, every 100 seconds.

  case MENU_DEBUG_OFF: { DeBug = 0; break; }           // DeBug off.
  case MENU_DEBUG_1:   { DeBug = DeBug ^ 1;   break; } // DeBug .
  case MENU_DEBUG_2:   { DeBug = DeBug ^ 2;   break; } // DeBug .
  case MENU_DEBUG_3:   { DeBug = DeBug ^ 4;   break; } // DeBug .
  case MENU_DEBUG_4:   { DeBug = DeBug ^ 8;   break; } // DeBug .
  case MENU_DEBUG_5:   { DeBug = DeBug ^ 16;  break; } // DeBug .
  case MENU_DEBUG_6:   { DeBug = DeBug ^ 32;  break; } // DeBug .
  case MENU_DEBUG_7:   { DeBug = DeBug ^ 64;  break; } // DeBug .
  case MENU_DEBUG_8:   { DeBug = DeBug ^ 128; break; } // DeBug .

  case SLIDER_1: { scaling1 = slider1->Value()/100.0; be_app->PostMessage(REFRESH_SCREEN); break; } // Scale input 1.
  case SLIDER_2: { scaling2 = slider2->Value()/100.0; be_app->PostMessage(REFRESH_SCREEN); break; } // Scale input 2.
  case SLIDER_3: { scaling3 = slider3->Value()/100.0; be_app->PostMessage(REFRESH_SCREEN); break; } // Scale input 3.
  case SLIDER_4: { scaling4 = slider4->Value()/100.0; be_app->PostMessage(REFRESH_SCREEN); break; } // Scale input 4.

  case SLIDER_5: { offset1 = slider5->Value(); be_app->PostMessage(REFRESH_SCREEN); break; } // Offset position of input 1.
  case SLIDER_6: { offset2 = slider6->Value(); be_app->PostMessage(REFRESH_SCREEN); break; } // Offset position of input 2.
  case SLIDER_7: { offset3 = slider7->Value(); be_app->PostMessage(REFRESH_SCREEN); break; } // Offset position of input 3.
  case SLIDER_8: { offset4 = slider8->Value(); be_app->PostMessage(REFRESH_SCREEN); break; } // Offset position of input 4.
  
  case SLIDER_9: { HorzScale  = slider9->Value(); be_app->PostMessage(REFRESH_SCREEN); break; } // Offset position of Display.
  case SLIDER_0: { HorzOffset1 = slider0->Value(); HorzOffset0 = HorzOffset1 * 32 + HorzOffset2; be_app->PostMessage(REFRESH_SCREEN); break; } // Magnify Display.
  case SLIDER_A: { HorzOffset2 = sliderA->Value(); HorzOffset0 = HorzOffset1 * 32 + HorzOffset2; be_app->PostMessage(REFRESH_SCREEN); break; } // Magnify Display.

  default: { BWindow::MessageReceived(message); break; } ;
  } }

void Digital_Scope_Window::FrameMoved(BPoint(New_Position)) { // Calculate new positions of window. 
 aWindow1_Rect.right  = aWindow1_Rect.right  + New_Position.x - aWindow1_Rect.left; aWindow1_Rect.left = New_Position.x;
 aWindow1_Rect.bottom = aWindow1_Rect.bottom + New_Position.y - aWindow1_Rect.top;  aWindow1_Rect.top  = New_Position.y;
 if (DeBug) { alert = new BAlert("DeBug","FrameMoved.",  "OK"); result = alert->Go(); } }

// Digital_Colour_Window.cpp
Digital_Colour_Window::Digital_Colour_Window(BRect frame) // Define 'Digital_Scope_Window' constructor call.
               : BWindow(frame,                           // Frame area of the window
                 "Colour Preferences",                    // Name of the window, appears in the Tab on top of the window.
                 B_TITLED_WINDOW,                         // Type of window (Look & Feel)
                 B_NOT_RESIZABLE|B_NOT_CLOSABLE|B_NOT_ZOOMABLE|B_ASYNCHRONOUS_CONTROLS) { // Behavior of the window (resize knob, etc)
 aColor  = new BColorControl(BPoint(10.0,10.0),B_CELLS_32x8,4,"Colour Control",new BMessage(COLOUR_CHANGED)); // Setup Bcolour control panel. 
 AddChild(aColor); //aColor->SetEnabled(true); aColor->MakeFocus(true); aColor->WindowActivated(true); B_CELLS_4x64  B_CELLS_8x32  B_CELLS_16x16  B_CELLS_64x4 B_CELLS_32x8 
 X=300.0; Y=25.0; // Position and width of button.
 AddChild(new BButton(BRect(X,Y,X+80.0,Y+10.0),"OK_Button","Ok", new BMessage(COLOUR_OK))); };

bool Digital_Colour_Window::QuitRequested() { // ::QuitRequested. Close button on window pressed. A Blooper call to Bwindows. 
 Minimize(true); return(true); } ; // Send a message to the global variable 'be_app' to tell the program to quit. Tell the system (Blooper) to close window.

// Digital_Colour_Window::MessageReceived.  Called when a message is received by our application.
void Digital_Colour_Window::MessageReceived(BMessage *message) {
// alert = new BAlert("DeBug","Unknown Digital_Colour_Window::MessageReceived().","OK"); result = alert->Go();
 switch(message->what) {

  case COLOUR_CHANGED: {
       // alert = new BAlert("DeBug","Color changed.  Colour Window","OK"); result = alert->Go();
       if (color == 1) { gridColor    = aColor->ValueAsColor(); }
       if (color == 2) { bulleyeColor = aColor->ValueAsColor(); }
       if (color == 3) { digitalColor = aColor->ValueAsColor(); }
       if (color == 4) { zeroColor    = aColor->ValueAsColor(); }
       if (color == 5) { analog1Color = aColor->ValueAsColor(); }
       if (color == 6) { analog2Color = aColor->ValueAsColor(); }
       if (color == 7) { analog3Color = aColor->ValueAsColor(); }
       if (color == 8) { analog4Color = aColor->ValueAsColor(); }
       if (color == 9) { displayColor = aColor->ValueAsColor(); }
       be_app->PostMessage(COLOUR_SET); be_app->PostMessage(REFRESH_SCREEN); break; }

  case COLOUR_OK: {
       color = 0; Minimize(true); be_app->PostMessage(COLOUR_SET); be_app->PostMessage(REFRESH_SCREEN); break; }
 
  default: { BWindow::MessageReceived(message); break; } ; } }

void Digital_Colour_Window::FrameMoved(BPoint(New_Position)) { // Calculate new positions of window. 
 aWindow2_Rect.right  = aWindow2_Rect.right  + New_Position.x - aWindow2_Rect.left; aWindow2_Rect.left = New_Position.x;
 aWindow2_Rect.bottom = aWindow2_Rect.bottom + New_Position.y - aWindow2_Rect.top;  aWindow2_Rect.top  = New_Position.y;
 if (DeBug) { alert = new BAlert("DeBug","FrameMoved.",  "OK"); result = alert->Go(); } }

// Digital_Scope_World.cpp  The main() function's only real job in a basic BeOS application is to create the BApplication object and run it.
int main(int, char**) {
 Digital_Scope_Application myApplication; myApplication.Run(); return(0); } ; // Create an Be_app instance?  Run the app thread.  Start to process messages.  Return 0 for no errors.

Digital_Scope_Application::Digital_Scope_Application() : BApplication(APP_SIGNATURE) { // Define 'Digital_Scope_Application' constructor call.
 for (n=0; n<1028; n++) { InputBuffer1[n] = 128; InputBuffer2[n] = 128; InputBuffer3[n] = 128; InputBuffer4[n] = 128; }
 if (find_directory (B_USER_SETTINGS_DIRECTORY, &file_path) == B_OK) {
    file_path.Append("D.A.Scope_Settings_0.91"); ref = open(file_path.Path(), O_RDONLY);
 if (ref >= 0) {
       // Debugging options.
       read(ref,(char *)&DeBug,sizeof(DeBug));
       // Last position of display and colour control windows.
       read(ref,(char *)&aWindow1_Rect,sizeof(aWindow1_Rect)); read(ref,(char *)&aWindow2_Rect,sizeof(aWindow2_Rect));
       // General user control variables.
       read(ref,(char *)&TriggerValue, sizeof(TriggerValue));  read(ref,(char *)&TriggerMode,  sizeof(TriggerMode));
       read(ref,(char *)&InputType,    sizeof(InputType));     read(ref,(char *)&ScanMode,     sizeof(ScanMode));
       read(ref,(char *)&BullsEye,     sizeof(BullsEye));      read(ref,(char *)&GridMode,     sizeof(GridMode));         
       read(ref,(char *)&Timing,       sizeof(Timing));        read(ref,(char *)&Source,       sizeof(Source));
       // Data display scaling and positioning options.
       read(ref,(char *)&scaling1,sizeof(scaling1)); read(ref,(char *)&offset1,sizeof(offset1));
       read(ref,(char *)&scaling2,sizeof(scaling2)); read(ref,(char *)&offset2,sizeof(offset2));
       read(ref,(char *)&scaling3,sizeof(scaling3)); read(ref,(char *)&offset3,sizeof(offset3));
       read(ref,(char *)&scaling4,sizeof(scaling4)); read(ref,(char *)&offset4,sizeof(offset4));
       // User display selected colours. 
       read(ref,(char *)&digitalColor,sizeof(digitalColor)); read(ref,(char *)&displayColor,sizeof(displayColor)); read(ref,(char *)&bulleyeColor,sizeof(bulleyeColor));
       read(ref,(char *)&analog1Color,sizeof(analog1Color)); read(ref,(char *)&analog2Color,sizeof(analog2Color)); read(ref,(char *)&gridColor,   sizeof(gridColor));
       read(ref,(char *)&analog3Color,sizeof(analog3Color)); read(ref,(char *)&analog4Color,sizeof(analog4Color)); read(ref,(char *)&zeroColor,   sizeof(zeroColor)); close(ref); }
    else {
       aWindow1_Rect.Set(40, 40, 900, 700); aWindow2_Rect.Set(100, 100, 500, 200); } } // Set the Screen Bpoints into the rectangle object(aRect) to define a default window.

 aView1_Rect.Set(10, 10, 850, 650); // Set the Window Bpoints into the rectangle object(aRect) to define a view.
 aView2_Rect.Set(20, 20, 820, 420); // Set the Window Bpoints to define a view 801 pixels by 401 pixels.
 aView3_Rect.Set(10, 10, 400, 100); // Set the Window Bpoints to define a view for color control view.

// Create a window and views to display scope.
 aWindow1 = new Digital_Scope_Window(aWindow1_Rect); // Invoke a new dynamically allocated window object by calling the constructor.
 aView1   = new Digital_Scope_View(aView1_Rect, "Scope_View" ) ; // Invoke a new dynamically allocated view object inside the window by calling the constructor. Plus supply a ID name for the view.
 aView2   = new Digital_Display_View(aView2_Rect,"Scope_Display'" ) ; // Invoke a new dynamically allocated view object inside the window by calling the constructor. Plus supply a ID name for the view.
 aWindow1->AddChild(aView1); aView1->AddChild(aView2); aWindow1->Show(); // Add views to window. Redraw window with the views attached.

// Create a colour control window and a view to display colour controls.
 aWindow2 = new Digital_Colour_Window(aWindow2_Rect); // Invoke a new dynamically allocated window object by calling the constructor.
 aView3   = new Digital_Colour_View(aView3_Rect,"Scope_Colour" ) ; // Invoke a new dynamically allocated view object inside the window by calling the constructor. Plus supply a ID name for the view.
 aWindow2->AddChild(aView3); aWindow2->Show(); aWindow2->Minimize(true); // Add view to window. Redraw window with the views attached. Minimize window.
 }; // Setup new windows, restore to old positions if possible.

// ::~Digital_Scope_Application
Digital_Scope_Application::~Digital_Scope_Application() {
 if (find_directory (B_USER_SETTINGS_DIRECTORY, &file_path, true) == B_OK) {
    file_path.Append("D.A.Scope_Settings_0.91"); ref = creat(file_path.Path(), 0777);
    if (ref >= 0) {
       // Debugging options.
       write(ref,(char *)&DeBug,sizeof(DeBug));
       // Last position of display and colour control windows.
       write(ref,(char *)&aWindow1_Rect,sizeof(aWindow1_Rect)); write(ref,(char *)&aWindow2_Rect,sizeof(aWindow2_Rect));
       // General user control variables.
       write(ref,(char *)&TriggerValue,sizeof(TriggerValue)); write(ref,(char *)&TriggerMode,sizeof(TriggerMode));
       write(ref,(char *)&InputType,   sizeof(InputType));    write(ref,(char *)&ScanMode,   sizeof(ScanMode));
       write(ref,(char *)&BullsEye,    sizeof(BullsEye));     write(ref,(char *)&GridMode,   sizeof(GridMode));         
       write(ref,(char *)&Timing,      sizeof(Timing));       write(ref,(char *)&Source,     sizeof(Source));
       // Data display scaling and positioning options.
       write(ref,(char *)&scaling1,sizeof(scaling1)); write(ref,(char *)&offset1,sizeof(offset1));
       write(ref,(char *)&scaling2,sizeof(scaling2)); write(ref,(char *)&offset2,sizeof(offset2));
       write(ref,(char *)&scaling3,sizeof(scaling3)); write(ref,(char *)&offset3,sizeof(offset3));
       write(ref,(char *)&scaling4,sizeof(scaling4)); write(ref,(char *)&offset4,sizeof(offset4));
       // User display selected colours. 
       write(ref,(char *)&digitalColor,sizeof(digitalColor)); write(ref,(char *)&displayColor,sizeof(displayColor)); write(ref,(char *)&bulleyeColor,sizeof(bulleyeColor));
       write(ref,(char *)&analog1Color,sizeof(analog1Color)); write(ref,(char *)&analog2Color,sizeof(analog2Color)); write(ref,(char *)&gridColor,   sizeof(gridColor));
       write(ref,(char *)&analog3Color,sizeof(analog3Color)); write(ref,(char *)&analog4Color,sizeof(analog4Color)); write(ref,(char *)&zeroColor,   sizeof(zeroColor)); close(ref); }
    else {
       alert = new BAlert("BAlert","Could not save program settings.","OK"); result = alert->Go(); } } }

// ::AboutRequested. Recieved About message.
void Digital_Scope_Application::AboutRequested() { // Displays alert with warning.
 alert = new BAlert("About BAlert","About Digital Scope 0.91.\nUses the parallel port to act as a digital or analog scope.","OK"); result = alert->Go(); }

// ::MessageReceived.  Handle incoming messages. 
void Digital_Scope_Application::MessageReceived(BMessage *message) {
 switch(message->what) {
  case B_SAVE_REQUESTED: {
       // BMessage: what = SAVE (0x53415645, or 1396790853)
       //     entry      directory, type='RREF', c=1, size=17, data[0]: device=4, directory=131072, name="home", path="/boot/home"
       //     entry           name, type='CSTR', c=1, size=5, data[0]: "test"
       File_Error = message->FindRef("directory", &file_ref); // Process entry_ref for directory file is save in.
       File_Error = b_entry.SetTo(&file_ref);                 // Transfer into a BEntry structure.
       b_entry.GetPath(&file_path);                           // Bpath object set from BEntry
       File_Error = message->FindString("name", &file_name);  // Process file name separate text string.
       file_path.Append(file_name);                           // append file name to path.       
       File_Error = File_Object.SetTo(file_path.Path(), B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE); // Set BFile object to filter.
       File_Error = File_Object.Write((void *)InputBuffer1, 1026); File_Error = File_Object.Write((void *)InputBuffer2, 1026); // Write file.
       File_Error = File_Object.Write((void *)InputBuffer3, 1026); File_Error = File_Object.Write((void *)InputBuffer4, 1026);
       File_Object.Unset(); be_app->PostMessage(REFRESH_SCREEN); } // Close File. Refresh display.

  case REFRESH_SCREEN: {
       aWindow1->Lock(); /* aView1->Invalidate(); */ aView2->Invalidate(); aWindow1->Unlock(); break; } // Forces a redraw of the window.
  case COLOUR_SET: {
       aWindow2->Lock(); aView3->Invalidate(); aWindow2->Unlock(); break; } // Forces a redraw of the colour control window.

  case APP_SLEEP:     { sleep(1); break; }                 // Put thread to sleep, one second delay.
  case APP_PULSE_OFF: { SetPulseRate(0L);         break; } // Sends a pulse message to BApplication to turn off Pulses.
  case APP_PULSE_0:   { SetPulseRate(100000L);    break; } // Sends a pulse message to BApplication, every .1 seconds.
  case APP_PULSE_1:   { SetPulseRate(1000000L);   break; } // Sends a pulse message to BApplication, every 1 seconds.
  case APP_PULSE_10:  { SetPulseRate(10000000L);  break; } // Sends a pulse message to BApplication, every 10 seconds.
  case APP_PULSE_100: { SetPulseRate(100000000L); break; } // Sends a pulse message to BApplication, every 100 seconds.

  case APP_COUNT_UP: { // Count up parallel output.
       Display = 1; aWindow1->Lock(); aView1->Invalidate(); aView2->Invalidate(); aWindow1->Unlock(); // Forces a redraw of the display.
       port = new ParPort(); port->Open();             // Create a new Parallel Port. Remember to open the port.
       scan = port->In(PC) & 223; port->Out(PC, scan); // Mask & Write to Control register for output on parallel port.
       for (count = 0; count<256; count++) {           // Output range 0-255 to parallel port.
           port->Out(PO, count); snooze(Timing); }     // Output 1 byte to the parallel port. Timing delay
       port->Close();  break; }                        // Finished, and to be safe, close ParPort.

  case APP_COUNT_DOWN: { // Count down parallel output.
       Display = 2; aWindow1->Lock(); aView1->Invalidate(); aView2->Invalidate(); aWindow1->Unlock(); // Forces a redraw of the display.
       port = new ParPort(); port->Open();             // Create a new Parallel Port. Remember to open the port.
       scan = port->In(PC) & 223; port->Out(PC, scan); // Mask & Write to Control register for output on parallel port.
       for (count = 0; count<256; count++) {           // Ouput range 255-0 to parallel port.
           port->Out(PO, 255-count); snooze(Timing); } // Output 1 byte to the parallel port. Timing delay
       port->Close();  break; }                        // Finished, and to be safe, close ParPort.

  case APP_READ_PORT: { // Read values off parallel port registers.
       port = new ParPort(); port->Open();               // Create a new Parallel Port. Remember to open the port.
       data1 = port->In(PO);                             // Input 1 byte from the Data Register.
       data2 = port->In(PS);                             // Input 1 byte from the Status Register. 
       data3 = port->In(PC);                             // Input 1 byte from the Control Register.
       Display = 3; be_app->PostMessage(REFRESH_SCREEN); // Forces a redraw of the display.
       port->Close(); break; }                           // Finished, and to be safe, close ParPort.

 case APP_WRITE_PORT: { // Write a value onto the parallel port's outputs.
      port = new ParPort(); port->Open();               // Create a new Parallel Port. Remember to open the port.
      scan = port->In(PC) & 223; port->Out(PC, scan);   // Mask & Write to Control register for output on parallel port.
      port->Out(PO, data);                              // Output 1 byte to the parallel port
      Display = 4; be_app->PostMessage(REFRESH_SCREEN); // Forces a redraw of the display.
      port->Close(); break; }                           // Finished, and to be safe, close ParPort.

 case APP_SCAN_PLOT: { // Collect data to Display
 // InputType = 0: 2 Direct digital inputs, 1 bit each from 1 byte read from the parallel port.
 // InputType = 1: 4 Direct digital inputs, 1 bit each from 1 byte read from the parallel port.
 // InputType = 2: 6 Direct digital inputs, 1 bit each from 1 byte read from the parallel port.
 // InputType = 3: 8 Direct digital inputs, 1 bit each from 1 byte read from the parallel port.
 // InputType = 4: Read 8 LED Bar ADC in, 9 possible values from the parallel port.
 // InputType = 5: 2 ADC analog inputs, 4 bits each from the parallel port.
 // InputType = 6: 1 ADC analog input, 1 byte from the parallel port.
      if (InputType < 7) {
         port = new ParPort(); port->Open(); port->Out(PC, port->In(PC) | 32); // Create new Parallel Port, open the port. Mask & Write Control register for input
         port->Out(PO, 255); TimeBase = system_time(); // Set all outputs lines high. Get BaseTime to time sampling.
         for (n=0; n<1025; n++) { // Fill data buffer.
             InputBuffer1[n] = port->In(PO); // Store 8 Bits, or 2 ADC Nibbles, or 1 ADC Byte.
             if (Timing) { TimeBase = TimeBase + Timing; snooze_until(TimeBase, B_SYSTEM_TIMEBASE); } } // Wait till next valid time slice to do sampling.
         port->Close(); Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Finished, and to be safe, close ParPort. Forces a redraw of the display.

      // Comparor FeedBack on pin 10.
      if (InputType == 7) { // 1 scanned analog input, 1 byte from the parallel port.
         port = new ParPort(); port->Open(); port->Out(PC, port->In(PC) & 223); // Create new Parallel Port, open the port. Mask & Write to Control register for output.
         TimeBase = system_time();            // Get BaseTime to time sampling.
         for (n=0; n<1025; n++) {             // Fill data buffer.
             flag1 = true;                    // Flag buffer not valid yet.
             for (scan=0; scan<255; scan++) { // Test range of values.
                 if (flag1) {                 // Continue testing only if no valid value already found.
                    port->Out(PO, scan);      // Output test value.
                    if (port->In(PS) & 64) { InputBuffer1[n] = scan; flag1 = false; } } } // Test if scan value is greater than signal's.
             if (flag1) { InputBuffer1[n] = 255; } // If comparor flag was never triggered set input to max. value.
             if (Timing) { TimeBase = TimeBase + Timing; snooze_until(TimeBase, B_SYSTEM_TIMEBASE); } } // Wait till next valid time slice to do sampling.
         port->Close(); Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Finished, and to be safe, close ParPort. Forces a redraw of the display.

      // Comparor FeedBack to pins 10, 12.
      if (InputType == 8) { // 2 scanned analog inputs, 1 byte from the parallel port.
         port = new ParPort(); port->Open(); port->Out(PC, port->In(PC) & 223); // Create new Parallel Port, open the port. Mask & Write to Control register for output.
         TimeBase = system_time();            // Get BaseTime to time sampling.
         for (n=0; n<1025; n++) {             // Fill data buffers.
             flag1 = true; flag2 = true;      // Flag buffers not valid yet.
             for (scan=0; scan<255; scan++) { // Test range of values.
                 port->Out(PO, scan); data = port->In(PS); // Output test value. Read in comparor flags.
                 if (flag1) { if (data & 64) { InputBuffer1[n] = scan; flag1 = false; } }   // If buffer not already set, test if scan value is greater than signal's. If yes, set buffer & flag.
                 if (flag2) { if (data & 32) { InputBuffer2[n] = scan; flag2 = false; } } } // If buffer not already set, test if scan value is greater than signal's. If yes, set buffer & flag.
             if (flag1) { InputBuffer1[n] = 255; } // If comparor flag was never triggered set input to max. value.
             if (flag2) { InputBuffer2[n] = 255; } // If comparor flag was never triggered set input to max. value.
             if (Timing) { TimeBase = TimeBase + Timing; snooze_until(TimeBase, B_SYSTEM_TIMEBASE); } } // Wait till next valid time slice to do sampling.
         port->Close(); Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Finished, and to be safe, close ParPort. Forces a redraw of the display.

      // Comparer FeedBack to pins 10, 12, 13, 15.
      if (InputType == 9) { // 4 scanned analog inputs, 1 byte from the parallel port.
         port = new ParPort(); port->Open(); port->Out(PC, port->In(PC) & 223); // Create new Parallel Port, open the port. Mask & Write to Control register for output.
         TimeBase = system_time(); // Get BaseTime to time sampling.
         for (n=0; n<1025; n++) { // Fill data buffers.
             flag1 = true; flag2 = true; flag3 = true; flag4 = true; // Flag buffers not valid yet.
             for (scan=0; scan<255; scan++) { // Test range of values.
                 port->Out(PO, scan); data = port->In(PS); // Output test value. Read in comparor flags.
                 if (flag1) { if (data & 64) { InputBuffer1[n] = scan; flag1 = false; } }   // If buffer not already set, test if scan value is greater than signal's. If yes, set buffer & flag.
                 if (flag2) { if (data & 32) { InputBuffer2[n] = scan; flag2 = false; } }   // If buffer not already set, test if scan value is greater than signal's. If yes, set buffer & flag.
                 if (flag3) { if (data & 16) { InputBuffer3[n] = scan; flag3 = false; } }   // If buffer not already set, test if scan value is greater than signal's. If yes, set buffer & flag.
                 if (flag4) { if (data &  8) { InputBuffer4[n] = scan; flag4 = false; } } } // If buffer not already set, test if scan value is greater than signal's. If yes, set buffer & flag.
             if (!flag1) { InputBuffer1[n] = 255; } // If comparor flag was never triggered set input to max. value.
             if (!flag2) { InputBuffer2[n] = 255; } // If comparor flag was never triggered set input to max. value.
             if (!flag3) { InputBuffer3[n] = 255; } // If comparor flag was never triggered set input to max. value.
             if (!flag4) { InputBuffer4[n] = 255; } // If comparor flag was never triggered set input to max. value.
             if (Timing) { TimeBase = TimeBase + Timing; snooze_until(TimeBase, B_SYSTEM_TIMEBASE); } } // Wait till next valid time slice to do sampling.
         port->Close(); Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } // Finished, and to be safe, close ParPort. Forces a redraw of the display.

       if (InputType == 10) {
          port = new ParPort(); port->Open(); port->Out(PC, port->In(PC) | 32); // Create new Parallel Port, open the port. Mask & Write Control register for input
          port->Out(PO, 255); TimeBase = system_time();  // Set all outputs lines high. Get BaseTime to time sampling.
          for (n=0; n<1025; n++) { // Fill data buffers.
              InputBuffer1[n] = port->In(PO); // Stores lower 8 bits of a 10 LEDs BAR ADC
              InputBuffer2[n] = port->In(PS); // Store upper 2 bits of a 10 LEDs BAR ADC
              if (Timing) { TimeBase = TimeBase + Timing; snooze_until(TimeBase, B_SYSTEM_TIMEBASE); } }
          port->Close(); Display = 5; be_app->PostMessage(REFRESH_SCREEN); break; } }

  } } // End of messages received.

// ::RefsReceived.  Handle a refs received message. The message specifies where to get it (see BFilePanel).
void Digital_Scope_Application::RefsReceived(BMessage *message) {
// BMessage: what = _RRC (0x5f525243, or 1599230531)
//     entry           refs, type='RREF', c=1, size=21, data[0]: device=4, directory=262368, name="testdata", path="/boot/home/testdata"
 File_Error = message->FindRef("refs", &file_ref);
 if (File_Error == B_NO_ERROR) {
    if (LoadFlag == true) { // True for save file.     
       File_Error = File_Object.SetTo(&file_ref, B_READ_ONLY); File_Error = File_Object.GetSize(&File_Size); // Set BFile object to a file, ReadOnly protects from changes. Get size of file.
       Bytes_Read = File_Object.Read((void *)InputBuffer1, 1026); Bytes_Read = File_Object.Read((void *)InputBuffer2, 1026); // Read file into buffers for display.
       Bytes_Read = File_Object.Read((void *)InputBuffer3, 1026); Bytes_Read = File_Object.Read((void *)InputBuffer4, 1026);
       File_Object.Unset(); LoadFlag = false; be_app->PostMessage(REFRESH_SCREEN); } } } // Close File. Refresh display.

// ::ReadyToRun.  A good place to set up windows, etc., just before your application is about to Run().
void Digital_Scope_Application::ReadyToRun() { }

// ::Pulse.  Provides a "heartbeat" for your application; a good place to blink cursors, etc.
void Digital_Scope_Application::Pulse() { } // Process a pulse.